@@ -5,6 +5,7 @@
 # backend_aml.py: support for running jobs under Azure Machine Learning
 
 import os
+from platform import python_version
 import shutil
 import logging
 import urllib.request
@@ -116,6 +117,9 @@ class AzureML(BackendBase):
 
             # we update each box's command (passing RUN_NAME as arg to __wrapped__.sh)
             script_part = "{} {} {}".format(os.path.basename(fn_wrapped), i, run_name)
+
+            # using "source" to the script is failing on ITP (Nov-4-2022, so changing it back to /bin/bash)
+            #sh_parts = ['source', script_part]
             sh_parts = ['/bin/bash', '--login', script_part]
 
             # pass sh_parts as a single argument to avoid wierd "arg": 1 problems with AML estimators
@@ -370,17 +374,21 @@ class AzureML(BackendBase):
         results_by_aml = {"Azure ML": results}
         return results_by_aml
 
-    def get_active_jobs(self, ws_name):
+    def get_active_jobs(self, ws_name, job_list=None):
         ''' return a list of job_id's running on this instance of Azure Batch '''
         db = self.store.get_database()
 
-        filter_dict = {
-            "username": self.username,
-            "compute": "aml",
-            "job_status": {
-                "$nin": ["created", "completed"]
+        if job_list:
+            filter_dict = {"job_id": {"$in": job_list}}
+        
+        else:
+            filter_dict = {
+                "username": self.username,
+                "compute": "aml",
+                "job_status": {
+                    "$nin": ["created", "completed"]
+                }
             }
-        }
 
         fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
 
@@ -399,7 +407,13 @@ class AzureML(BackendBase):
         cancel_results = []
 
         # get list of active jobs from batch
-        active_jobs = self.get_active_jobs(ws_name)
+        if box_name:
+            job_id = box_name.split("-")[0]
+            active_jobs = self.get_active_jobs(ws_name, [job_id])
+
+        else:
+            active_jobs = self.get_active_jobs(ws_name)
+
         console.diag("after get_active_jobs()")
 
         if active_jobs:
@@ -433,7 +447,7 @@ class AzureML(BackendBase):
 
         # we build service-based box names to have 3 parts
         job_id, service_name, node_index = box_name.split("-")
-        active_jobs = self.get_active_jobs(workspace)
+        active_jobs = self.get_active_jobs(workspace, [job_id])
         cancel_results = []
         if active_jobs:
             for job_record in active_jobs:
@@ -866,6 +880,15 @@ class AzureML(BackendBase):
                 if container_registry:
                     environment.docker.base_image_registry = container_registry
 
+                # let user specify additional docker options
+                other_args = args["docker_other_options"] if args["docker_other_options"] else []
+
+                if use_sudo:
+                    # specify additional docker run flags so that we can use blobfuse
+                    other_args += ["--privileged"]
+
+                environment.docker.arguments = other_args
+
                 if docker_image:
                     # override AML default base image with user-specified image
                     environment.docker.base_image = docker_image 
@@ -873,18 +896,13 @@ class AzureML(BackendBase):
                      environment.docker.base_image = self.get_default_docker_image(use_gpu)
 
                 # specify conda_packages and pip_packages here
+                python_version = args["python_version"]
+
                 conda = azureml.core.environment.CondaDependencies.create(pip_packages=pip_packages, 
-                    conda_packages=conda_packages)
+                    conda_packages=conda_packages, python_version=python_version)
 
                 environment.python.conda_dependencies = conda
 
-                if use_sudo:
-                    # specify additional docker run flags so that we can use blobfuse
-                    #docker_flags == "--privileged"
-                    environment.docker.arguments = ["--privileged"]
-                    #environment.docker.arguments = ["--cap-add", "SYS_ADMIN", "--device", "/dev/fuse"]
-                    #environment.docker.arguments = ["--cap-add", "SYS_ADMIN", "--device", "/dev/fuse", "--security-opt", "apparmor:unconfined"]
-
                 estimator = estimator_ctr(source_directory=code_dir, script_params=arg_dict, compute_target=actual_target, 
                     vm_size=vm_size, distributed_training=distributed_obj, 
                     entry_script=target_fn, node_count=node_count, max_run_duration_seconds=max_secs, environment_definition=environment)
@@ -946,6 +964,9 @@ class AzureML(BackendBase):
             node_id = "node" + str(i)
             service_info_by_node[node_id] = node_info
 
+            # update node_info record in database with service_info
+            self.store.database.update_node_info_with_service_info(workspace, job_id, i, node_info)
+
         fb.reset_feedback()
         fb.feedback("submitted")
 
@@ -1115,6 +1136,11 @@ class AzureML(BackendBase):
             aml_run_id = aml_run.id
             aml_run_number = aml_run.number
 
+            # set AML/ITP display_name 
+            display_name = args["display_name"]
+            display_name = utils.expand_xt_vars(display_name, run_id=run_name, node_index=node_index, args=args)
+            aml_run.display_name = display_name
+
         # copy to submit-logs
         utils.copy_data_to_submit_logs(args, self.serializable_trainer, "aml_submit.json")
 
@@ -1268,7 +1294,7 @@ class AzureML(BackendBase):
     def get_simple_status(self, status):
         # translates an AML status to a simple status (queued, running, completed)
 
-        queued = ["NotStarted", "Starting", "Provisioning", "Preparing", "Queued"]
+        queued = ["NotStarted", "Starting", "Provisioning", "Preparing", "Queued", "Paused"]
         running = ["Running", "Finalizing"]
         completed = ["CancelRequested", "Completed", "Failed", "Canceled", "NotResponding"]
 
@@ -1345,3 +1371,7 @@ class AzureML(BackendBase):
         self.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml_compute_logs/* {}".format(dest_dir))
         self.append(cmds, "cp -r $XT_ORIG_WORKDIR/azureml-logs/* {}".format(dest_dir))
         self.append(cmds, "cp -r $XT_ORIG_WORKDIR/logs/* {}".format(dest_dir))
+
+    # API call
+    def get_log_files_dir(self, args):
+        return "$XT_ORIG_WORKDIR/logs"
