@@ -33,6 +33,7 @@ class BackendBase(implements(BackendInterface)):
         self.capture_setup_cmds = False
         self.echo_cmds = True
         self.default_docker_image = ""
+        self.mounted_drives = []
 
         # TODO: change this to rely on API for this info
         self.add_time = self.get_name() in ["pool", "batch", "aml"]
@@ -262,6 +263,8 @@ class BackendBase(implements(BackendInterface)):
             readonly_opt = "-o ro" if readonly else ""
             nonempty_opt = "-o nonempty" if nonempty else ""
 
+            self.mounted_drives.append(mnt_dir)
+
             self.append(cmds, "{}mkdir {} -p".format(sudo, mnt_dir))
 
             if use_username:
@@ -802,6 +805,8 @@ class BackendBase(implements(BackendInterface)):
             for cmd in pp:
                 self.append(cmds, cmd)
 
+            self.cd_back_to_setup_dir(cmds)
+
     def set_gen_for_windows(self, for_windows: bool, args: List[str]):
         docker_cmd = args["docker_cmd"]
         docker_is_windows = args["docker_is_windows"]
@@ -843,11 +848,8 @@ class BackendBase(implements(BackendInterface)):
             self.mounting_enabled = False
 
         docker_cmd = args["docker_cmd"]
-        docker_name = args["docker"]
-        job_id = args["job_id"]
-
         cmds = []
-        
+
         if gen_for_windows:
             cmds.append("@echo off")
 
@@ -1016,17 +1018,6 @@ class BackendBase(implements(BackendInterface)):
         we cannot rely on blobfuse mapping, and we know we have xtlib installed.
         '''
 
-        #fn_stdout_aml = "azureml-logs/00_f5cca3f8f56b6f68d8499ecd4520d4a8-master-0_stdout.txt"
-        # debug (ensure azureml-logs is present on ITP runs)
-        #self.append(cmds, "cd $XT_ORIG_WORKDIR")
-        #self.append_dir(cmds, "/var/log/compute", True)
-        #self.append_dir(cmds, "logs", True)
-
-        # these are empty for ITP jobs
-        #self.append_dir(cmds, "$AZUREML_RUN_CONFIGURATION")
-        #self.append_dir(cmds, "./azureml-logs", True)
-        #self.append_dir(cmds, "./outputs", True)
-
         self.append_title(cmds, "UPLOAD service and XT logs to job storage:")
 
         # workaround for "$jobs" not resolving correctly on compute node
@@ -1074,6 +1065,13 @@ class BackendBase(implements(BackendInterface)):
                 cmd = "xt upload {} {} --feedback=0".format(src_after, store_after)
                 self.append(cmds, cmd)
 
+            # remove all mounted drives
+            if self.mounted_drives:
+                self.append_title(cmds, "Unmount Blobfuse drives")
+                
+                for md in self.mounted_drives:
+                    self.append(cmds, "fusermount -u -q {} && rm -rf {}".format(md, md))
+
         # if True:
         #     # DEBUG: keep each node alive for 7 days for Azure Batch team to debug it
         #     self.append(cmds, "sleep 7d")
@@ -1105,7 +1103,9 @@ class BackendBase(implements(BackendInterface)):
         if not self.gen_for_windows:
 
             # show name of system
-            self.append(cmds, "hostnamectl")
+            #self.append(cmds, "hostnamectl")
+            # this is more reliable for singularity, batch
+            self.append(cmds, "cat /etc/os-release")
 
             self.append_title(cmds, "CPU report:", True)
             self.append(cmds, "lscpu")
@@ -1119,10 +1119,10 @@ class BackendBase(implements(BackendInterface)):
 
             # show GPU installed
             #self.append(cmds, "sudo lshw -C display | grep product")
-            self.append(cmds, "sudo lshw -C display")
+            #self.append(cmds, "sudo lshw -C display")
 
             # show GPU installed
-            self.append(cmds, "lspci -v")
+            #self.append(cmds, "lspci -v")
     
             # several techinques to show version of NVIDIA DRIVERS
             self.append(cmds, "whereis nvidia")
@@ -1277,7 +1277,7 @@ class BackendBase(implements(BackendInterface)):
                 fn_wrapped = self.write_cmds_to_file(pull_cmds, bootstrap_dir, self.is_windows, constants.FN_DOCKER_PULL_BAT, constants.FN_DOCKER_PULL_SH)
                 utils.copy_to_submit_logs(args, fn_wrapped)
 
-                pull_cmd = '''timeout {} bash -c "until {}; do '''.format(timeout, constants.FN_DOCKER_PULL_SH) + \
+                pull_cmd = '''timeout {} bash -c "until bash {}; do '''.format(timeout, constants.FN_DOCKER_PULL_SH) + \
                     '''echo XT: docker pull+test failed.  retrying...; sleep 60; done" '''
 
                 self.append(docker_cmds, pull_cmd)
@@ -1370,9 +1370,12 @@ class BackendBase(implements(BackendInterface)):
         cmd = text.format(external_controller_port, is_aml, multi_run_hold_open)
         return cmd
 
-    def add_pip_packages(self, cmds, pip_packages):
+    def add_pip_packages(self, cmds, pip_packages, use_legacy_resolver=False):
         if pip_packages:
             cmd = "pip install" if self.gen_for_windows else "pip install --user"
+            if use_legacy_resolver:
+                cmd += " --use-deprecated=legacy-resolver"
+
             # NOTE: double quotes around package names cause error on linux
             for pp in pip_packages:
                 cmd += ' {}'.format(pp)
@@ -1386,8 +1389,15 @@ class BackendBase(implements(BackendInterface)):
                 cmd += ' {}'.format(cp)
             self.append(cmds, cmd, log="conda_install")
 
-    def add_setup_cmds(self, cmds, remove_zip=True, 
-            gen_for_windows=False, args=None):
+
+    def cd_back_to_setup_dir(self, cmds):
+        #cd back to setup directory 
+        if self.gen_for_windows:
+            self.append(cmds, "cd %XT_CWD%")
+        else:
+            self.append(cmds, "cd $XT_CWD")
+
+    def add_setup_cmds(self, cmds, remove_zip=True, gen_for_windows=False, args=None):
 
         pre_cmds = args["pre_cmds"]
         if pre_cmds:
@@ -1395,6 +1405,8 @@ class BackendBase(implements(BackendInterface)):
             for cmd in pre_cmds:
                 self.append(cmds, cmd)
 
+            self.cd_back_to_setup_dir(cmds)
+
         # KISS: just insist that user specify PYTHONPATH in SETUP (not env vars)
         preserve_python_path=False
 
@@ -1435,7 +1447,8 @@ class BackendBase(implements(BackendInterface)):
         # install PIP-PACKAGES (xtlib and user-specified others)
         pip_packages = args["pip_packages"]
         if pip_packages:
-            self.add_pip_packages(cmds, pip_packages)
+            use_legacy_resolver = args["use_legacy_resolver"]
+            self.add_pip_packages(cmds, pip_packages, use_legacy_resolver)
 
         if pip_freeze:
             self.append(cmds, "pip freeze", log="final_pip_freeze")
