@@ -374,7 +374,7 @@ class AzureBatch(BackendBase):
                 using_hp, exper_name, static_run_cmds, args):
 
             # process a subset of all jobs on this thread
-            for run_data_list in job_runs:
+            for i, run_data_list in enumerate(job_runs):
                 
                 node_index = run_data_list[0]["node_index"]
 
@@ -394,9 +394,15 @@ class AzureBatch(BackendBase):
                     node_id = "node" + str(node_index)
                     batch_job_id = self.make_batch_job_id(workspace, job_id)
 
-                    service_info_by_node[node_id] = {"workspace": workspace, "job_id": job_id, "batch_job_id": batch_job_id, 
+                    node_info = {"workspace": workspace, "job_id": job_id, "batch_job_id": batch_job_id, 
                         "node_id": node_id, "task_index": node_index, "merge_batch_logs": merge_batch_logs}
 
+                    service_info_by_node[node_id] = node_info
+
+                # update node_info record in database with service_info
+                self.store.database.update_node_info_with_service_info(workspace, job_id, node_index, node_info)
+                #print("updated node_info: job_id='{}', node_index='{}'".format(job_id, node_index))
+
         # create each Batch Task on a worker thread
         max_run_workers = args["max_workers"]
         utils.run_on_threads(thread_worker, job_runs_list, max_run_workers, [job_id, num_nodes, use_low_pri, 
@@ -1025,11 +1031,11 @@ class AzureBatch(BackendBase):
 
         if task.state != "completed":
             try:
-                self.batch_client.task.terminate(job_id, task.id)     
+                self.batch_client.task.terminate(batch_job_name, task.id)     
                 console.print("azure-batch task terminated: {}.{}".format(job_id, task.id))
                 task_killed = True
-            except:
-                pass
+            except BaseException as ex:
+                print("exception trying to cancel job '{}', task: '{}': {}".format(job_id, task.id, ex.message))
 
         # kill the NODE itself
         # TODO: we need the resource_group to make this call
@@ -1476,7 +1482,16 @@ class AzureBatch(BackendBase):
 
             # this call can take up to 15 secs
             if not task:
-                task = self.get_node_task(service_node_info)
+                # sometimes this fails with "Request date headers too old" error
+                for i in range(5):
+                    try:
+                        task = self.get_node_task(service_node_info)
+                    except BaseException as ex:
+                        console.print("monitor exception: {}".format(ex))
+                        time.sleep(3)
+
+                if not task:
+                    errors.service_error("fatal Azure Batch communication failure")
 
             node_info = task.node_info
             batch_status = task.state.value
@@ -1622,3 +1637,9 @@ class AzureBatch(BackendBase):
 
         return {"user": user_name, "pw": pw, "ip_addr": ip_addr, "port": port}
 
+    # API call
+    def get_log_files_dir(self, args):
+        docker_cmd = args["docker_cmd"]
+
+        path = "/usr/logs" if docker_cmd else "$XT_ORIG_WORKDIR/.."
+        return path
