@@ -45,13 +45,6 @@ class ITP(AzureML):
         estimator.run_config.cmk8scompute = k8sconfig
 
   # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
-
-        self.append(cmds, "mkdir -p {}".format(dest_dir))
-        self.append(cmds, "cp -r -v /var/log/compute/00_stdout.txt {}".format(dest_dir))
-        #self.append(cmds, "cp -r -v /var/log {}".format(dest_dir))
-        self.append(cmds, "cp -r -v $XT_ORIG_WORKDIR/logs {}".format(dest_dir))
-
     def create_wrapper_script(self, cmd_parts, snapshot_dir, store_data_dir, data_action, 
             data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
              sudo_available, username, setup, use_allow_other, args):
@@ -121,3 +114,14 @@ class ITP(AzureML):
         # self.append(cmds, "sudo chmod +x $xt_file")
         # self.append(cmds, "xt --version")
         # self.append(cmds, "python $xt_file --version")
+
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+
+        self.append(cmds, "mkdir -p {}".format(dest_dir))
+        self.append(cmds, "cp -r -v /var/log/compute/00_stdout.txt {}".format(dest_dir))
+        #self.append(cmds, "cp -r -v /var/log {}".format(dest_dir))
+        self.append(cmds, "cp -r -v $XT_ORIG_WORKDIR/logs {}".format(dest_dir))
+
+# API call
+    def get_log_files_dir(self, args):
+        return "var/log/compute"
