@@ -42,18 +42,6 @@ class Singularity(AzureML):
         '''
         return True   
         
-    # API call
-    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
-
-        self.append(cmds, "mkdir -p {}".format(dest_dir))
-
-        # copy known singularity log directories 
-        for log_dir in ["azureml-logs", "system_logs", "user_logs"]:
-            self.append(cmds, "cp -r -v $AZUREML_CR_EXECUTION_WORKING_DIR_PATH/{} {}".format(log_dir, dest_dir))
-
-        # this is now copy in user_logs above
-        #self.append(cmds, "cp -r -v $AZUREML_CR_EXECUTION_WORKING_DIR_PATH/user_logs/std_log.txt {}".format(dest_dir))
-
     # API call
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, env_vars, args):
 
@@ -91,7 +79,7 @@ class Singularity(AzureML):
             old_registry = old_env.docker.base_image_registry  
 
             container_registry, image_url, sing_dict = self.get_docker_container_registry(args)
-            sing_wrap = sing_dict["sing_wrap"]
+            sing_wrap = utils.safe_value(sing_dict, "sing_wrap")
 
             if sing_wrap:
                 # wrap our docker image with a singularity-compliant image
@@ -376,3 +364,22 @@ class Singularity(AzureML):
             "service_status": aml_status}
 
 
+    # API call
+    def add_service_log_copy_cmds(self, cmds, dest_dir, args):
+
+        self.append(cmds, "mkdir -p {}".format(dest_dir))
+
+        # copy known singularity log directories 
+        for log_dir in ["user_logs"]:    # "azureml-logs", "logs",
+            from_dir = "$AZUREML_CR_EXECUTION_WORKING_DIR_PATH/{}".format(log_dir)
+
+            self.append(cmds, "ls -R -lt {}/*".format(from_dir), echo=True)
+            self.append(cmds, "cp -r -v {}/* {}".format(from_dir, dest_dir))
+
+        # this is now copy in user_logs above
+        #self.append(cmds, "cp -r -v $AZUREML_CR_EXECUTION_WORKING_DIR_PATH/user_logs/std_log.txt {}".format(dest_dir))
+
+    # API call
+    def get_log_files_dir(self, args):
+        # singularity adds the "userlogs" at the end
+        return "$AZUREML_CR_EXECUTION_WORKING_DIR_PATH/user_logs"
