@@ -291,6 +291,8 @@ class CmdCore():
         # fri, lri = self.compute_run_indexes_for_node(total_run_count, node_count, node_count-1)
         # assert lri==total_run_count-1
 
+        context.log_file_uploads = args["log_file_uploads"]
+
         context.first_run_index = first_run_index
         context.last_run_index = last_run_index
 
@@ -396,8 +398,25 @@ class CmdCore():
         context.xtlib_capture = args["xtlib_upload"]
 
         # for mirroring files to grok server or storage
+        mirror_request_list = []
+        mf = args["mirror_files"]
+        if mf:
+            mirror_request_list.append( {"local_dir": mf, "dest_type": "run", "dest_folder": None} )
+
+        if args["mirror_log_files"]:
+            # add the backend node console log file(s)
+            backend = self.create_backend(args["target"], compute_def, username=context.username)
+            log_files_dir = backend.get_log_files_dir(args)
+            log_files_wc = "{}/*.txt".format(log_files_dir)
+            mirror_request_list.append( {"local_dir": log_files_wc, "dest_type": "node", 
+                "dest_folder": "service_logs"} )
+
+            # add the child/parent run log files dir
+            mirror_request_list.append( {"local_dir": "service_logs/*.txt", 
+                "dest_type": "run", "dest_folder": "service_logs"} )
+
         context.mirror_dest = args["mirror_dest"]
-        context.mirror_files =  args["mirror_files"]
+        context.mirror_request_list = mirror_request_list
         context.mirror_delay_mins =  self.config.get("logging", "mirror-delay-mins")
         context.show_mirror_calls =  self.config.get("logging", "show-mirror-calls")
         context.grok_server = None   # args["grok_server"]
