@@ -11,7 +11,7 @@ here, check to see if you need to also change it in psm.py.
 
 # Sync the BUILD string changes with: setup.py, CHANGELOG.md
 # Also, consider setting pytorch-xtlib docker image (in facotry_config.yaml) to new version just before releasing new XT
-BUILD = "version: 0.0.277, build: Aug-26-2022"
+BUILD = "version: 0.0.289, build: Nov-04-2022"
     
 INFO_CONTAINER_V1 = "xt-store-info"
 INFO_CONTAINER_V2 = "00-store-info"
@@ -164,7 +164,7 @@ PSM_QUEUE = "~/.xt/psm_queue"
 CWD = "~/.xt/cwd"
 PSM = "psm.py"
 PSMLOG = "psm.log"
-PSM_LOGDIR = "~/.xt/psm_logs"
+PSM_LOGDIR = "~/.xt/cwd/logs"
 CURRENT_RUNNING_ENTRY = "__current_running_entry__.txt"
 
 # hyperparameter YAML property names
