@@ -65,7 +65,7 @@ queue_lock = Lock()
 runs_lock = Lock()            
 rundir_lock = Lock()
 
-def read_from_pipe(pipe, run_info, runner, stdout_is_text):
+def read_from_stdout_pipe(stdout_pipe, run_info:RunInfo, controller, stdout_is_text:bool):
 
     last_run_exception = None
     in_traceback = False
@@ -78,9 +78,9 @@ def read_from_pipe(pipe, run_info, runner, stdout_is_text):
 
         while True:
             if stdout_is_text:
-                text_msg = pipe.readline()
+                text_msg = stdout_pipe.readline()
             else:
-                binary_msg = pipe.readline()
+                binary_msg = stdout_pipe.readline()
                 text_msg = binary_msg.decode("utf-8")
 
             if len(text_msg) == 0 or run_info.killing:
@@ -104,20 +104,28 @@ def read_from_pipe(pipe, run_info, runner, stdout_is_text):
             run_info.process_run_output(text_msg)
 
         # run post-processing
-        runner.exit_handler(run_info, called_from_thread_watcher=True, 
+        controller.exit_handler(run_info, called_from_thread_watcher=True, 
             last_run_exception=last_run_exception, traceback_lines=traceback_lines)
 
+        # remove the store instance for this watcher thread
+        controller.remove_store()
+
     except BaseException as ex:
 
-        logger.exception("Error in controller.read_from_pipe, ex={}".format(ex))
-        console.print("** Exception during read_from_pipe(): ex={}".format(ex))
+        logger.exception("Error in controller.read_from_stdout_pipe, ex={}".format(ex))
+        console.print("** Exception during read_from_stdout_pipe(): ex={}".format(ex))
 
         # for debugging print stack track
         traceback.print_exc()
 
         # log end of controller to JOB
-        runner.store.log_node_event(run_info.workspace, runner.job_id, runner.node_id, "node_error", 
-            {"node_id": runner.node_id, "run": runner.parent_run_name, "exception": str(ex)})
+        store = controller.get_store()
+        if store:
+            try:
+                store.log_node_event(run_info.workspace, controller.job_id, controller.node_id, "node_error", 
+                    {"node_id": controller.node_id, "run": controller.parent_run_name, "exception": str(ex)})
+            except:
+                pass
 
         # give dev a chance to read the error before exiting (if viewing live log)
         console.print("controller sleeping 30 secs before exiting ...")
@@ -177,10 +185,11 @@ class XTController(rpyc.Service):
         self.runs = {}       # all runs that we know about (queued, spawing, running, or completed)
         self.queue = []      # runs that are waiting to run (due to concurrent)
         self.mirror_workers = []
-        self.running_jobs = {}
+        self.running_job = None
         self.next_run_list_index = 0
         self.stop_processing_runs = False
         self.total_run_count = 0
+        self.store_instances = {}
 
         self.box_secret = os.getenv("XT_BOX_SECRET")
         self.node_id = os.getenv("XT_NODE_ID")
@@ -265,6 +274,36 @@ class XTController(rpyc.Service):
             #queue_worker.daemon = True          # don't wait for this thread before exiting
             queue_worker.start()
     
+    def get_store(self):
+        '''
+        Ensure we don't share store instances between threads (each thread will use their own dedicated instance)
+        '''
+        thread_id = threading.get_ident()
+        store = None
+
+        if self.mrc_context:
+            if not thread_id in self.store_instances:
+                # make a new instance of Store for this thread
+                log_info("creating a new store instance for thread=", thread_id)
+
+                store =  store_from_context(self.mrc_context)
+                self.store_instances[thread_id] = store
+            else:
+                store = self.store_instances[thread_id]
+
+        return store
+
+    def remove_store(self):
+        '''
+        Remove the allocated store instance for the current thread
+        '''
+        thread_id = threading.get_ident()
+
+        if self.mrc_context:
+            if thread_id in self.store_instances:
+                log_info("removing store instance for thread=", thread_id)
+                del self.store_instances[thread_id]
+        
     def process_multi_run_context(self, multi_run_context_fn):
         # read the cmd and context from the file
 
@@ -296,8 +335,7 @@ class XTController(rpyc.Service):
         self.last_run_index = context.last_run_index
 
         # cache store for later use (don't want storge/db traffic needed for subsequent creations)
-        store = store_from_context(context)
-        self.store = store
+        store = self.get_store()
         db = store.database
 
         # is this node restarting after a low-priority preemption (or a simulated one)?
@@ -362,14 +400,16 @@ class XTController(rpyc.Service):
         
         # simple scheduling
         self.run_indexes = list(range(context.first_run_index, 1 + context.last_run_index))
-        log_info("full run_indexes", self.run_indexes)
+        if context.schedule != "dynamic":
+            log_info("full run_indexes", self.run_indexes)
 
         if self.node_restart:
             # remove any runs that have completed (don't rely on v1/v2 child run name differences)
             filter_dict = {"ws_name": self.ws_name, "job_id": self.job_id, "node_index": self.node_index, 
                 "end_id": {"$exists": True}}
 
-            completed_run_docs = self.store.database.get_info_for_runs(self.ws_name, filter_dict, {"run_index": 1})
+            store = self.get_store()
+            completed_run_docs = store.database.get_info_for_runs(self.ws_name, filter_dict, {"run_index": 1})
             completed_run_indexes = [doc["run_index"] for doc in completed_run_docs]
 
             if completed_run_indexes:
@@ -378,6 +418,9 @@ class XTController(rpyc.Service):
                 log_info("NODE RESTART", "removed indexes={}".format(completed_run_indexes))
                 log_info("new run_indexes", self.run_indexes)
 
+        # mirrow the node-level files requested (usually just the node console log file(s))
+        self.start_mirror_workers_for_node(store, context)
+
         # queue up first job
         self.queue_job_core(context, cmd_parts, aml_run=aml_run)
 
@@ -440,9 +483,9 @@ class XTController(rpyc.Service):
 
         # stop logging (and free log file for deletion in below code)
         logging.shutdown()        
+        store = self.get_store()
 
         if context:
-            store = self.store
             node_id = os.getenv("XT_NODE_ID")
 
             # write XT event log to job store AFTER
@@ -458,12 +501,12 @@ class XTController(rpyc.Service):
                 #console.print("finished upload of CONTROLLER log")
 
         log_title("XT controller: db call stats")
-        if self.store.database:
-            self.store.database.print_call_stats()
+        if store:
+            store.database.print_call_stats()
 
-        # log end of controller to JOB
-        self.store.log_node_event(self.ws_name, self.job_id, self.node_id, 
-            "node_end", {"node_id": self.node_id, "run": self.parent_run_name})
+            # log end of controller to JOB
+            store.log_node_event(self.ws_name, self.job_id, self.node_id, 
+                "node_end", {"node_id": self.node_id, "run": self.parent_run_name})
 
         # # give other threads time to wrapup the processing of their runs before
         # # we exit
@@ -496,31 +539,38 @@ class XTController(rpyc.Service):
             self.restarting = False
         else:
             # prepare to shut down
+            context = None
+
             if self.runs:
                 first_run = list(self.runs.values())[0]
                 context = first_run.context
-                store = self.store
 
-                for job_id, alive in self.running_jobs.items():
-                    if alive:
-                        self.running_jobs[job_id] = False
+                job_id = self.running_job
+                if job_id:
+                    self.running_job = None
 
-                        # tell db this job node is exiting
-                        console.diag("calling db job_node_exit: job_id={}".format(job_id))
+            store = self.get_store()
+            job_id = self.job_id
+            ws_id = self.ws_name
+        
+            # tell db this job node is exiting
+            console.diag("calling db job_node_exit: job_id={}".format(job_id))
 
-                        db = store.get_database()
-                        if db:
-                            db_retries, storage_retries = run_errors.count_node_errors()
+            # always call job_node_exit() and node_end() before exiting
+            db = store.get_database()
+            if db:
+                db_retries, storage_retries = run_errors.count_node_errors()
 
-                            # log stats for job
-                            db.job_node_exit(context.ws, job_id)
+                # log stats for job
+                db.job_node_exit(ws_id, job_id)
 
-                            # log stats for node 
-                            utils.log_info("NODE db_retries", db_retries)
-                            utils.log_info("NODE storage_retries", storage_retries)
+                # log stats for node 
+                utils.log_info("NODE db_retries", db_retries)
+                utils.log_info("NODE storage_retries", storage_retries)
+
+                db.node_end(ws_id, job_id, self.node_index, db_retries, 
+                    storage_retries, self.app_start_str)
 
-                            db.node_end(context.ws, self.job_id, self.node_index, db_retries, 
-                                storage_retries, self.app_start_str)
             else:
                 context = None
 
@@ -639,7 +689,7 @@ class XTController(rpyc.Service):
             
             # should parent spawn new child?
             context = run_info.context
-            store = self.store
+            store = self.get_store()
 
             # get index of next run for our node storage
             #console.print("self.next_run_list_index=", self.next_run_list_index)
@@ -658,7 +708,7 @@ class XTController(rpyc.Service):
 
                 else:
                     # let database allocate next job-level run_index
-                    run_index =  self.store.database.get_next_run_index(context.ws, context.job_id)
+                    run_index =  store.database.get_next_run_index(context.ws, context.job_id)
                     entry = {"status": "unstarted", "run_index": run_index}
 
                     if self.total_run_count and run_index >= self.total_run_count:
@@ -722,25 +772,25 @@ class XTController(rpyc.Service):
         with runs_lock:
             self.runs[key] = run_info
 
-    def run_template(self, run_info, run_index, entry):
-        run_name = run_info.run_name
+    def run_template(self, parent_ri:RunInfo, run_index:int, entry:dict):
+        parent_run_name = parent_ri.run_name
 
-        # ensure PARENT has a rundir (so it can log its own output)
-        if not run_info.rundir:
-            rundir, rundir_index = self.allocate_rundir(run_name)
-            run_info.rundir = rundir
+        # ensure PARENT run has a rundir (so it can log its own output)
+        if not parent_ri.rundir:
+            rundir, rundir_index = self.allocate_rundir(parent_run_name)
+            parent_ri.rundir = rundir
 
-            # set up a console output file
-            console_fn = rundir + "/output/console.txt"
-            run_info.set_console_fn(console_fn)
+            # assign a console output file to the PARENT
+            console_fn = rundir + "/service_logs/stdboth.txt"
+            parent_ri.set_console_fn(console_fn)
 
         # create a parent log event for "spawning"
-        context = run_info.context
-        store = self.store
-        store.log_run_event(context.ws, run_name, "status-change", {"status": "spawning"}, job_id=self.job_id)  
+        context = parent_ri.context
+        store = self.get_store()
+        store.log_run_event(context.ws, parent_run_name, "status-change", {"status": "spawning"}, job_id=self.job_id)  
 
         # spawn child run from template
-        child_info = self.spawn_child(run_info, run_index, entry)
+        child_info = self.spawn_child(parent_ri, run_index, entry)
         if child_info:
 
             # add to runs
@@ -749,9 +799,9 @@ class XTController(rpyc.Service):
             # start normal run of CHILD
             self.start_local_run(child_info, cmd_parts=child_info.cmd_parts)
 
-            if run_info.status == "queued":
+            if parent_ri.status == "queued":
                 # create a parent log event for "spawing"
-                store.log_run_event(context.ws, run_name, "status-change", {"status": "queued"})  
+                store.log_run_event(context.ws, parent_run_name, "status-change", {"status": "queued"})  
         else:
             self.stop_processing_runs = True
             
@@ -789,7 +839,7 @@ class XTController(rpyc.Service):
         # update context with new cmd
         context.cmd_parts = utils.cmd_split(cmd)    
 
-        store = self.store
+        store = self.get_store()
 
         log_info("spawn_child, parent", parent.run_name)
         log_info("child run_index", run_index)
@@ -802,7 +852,7 @@ class XTController(rpyc.Service):
             arg_dict = self.hparam_search.process_child_hparams(run_index_name, store, context, parent)
             if arg_dict:
                 # now its safe to create the child name
-                child_name = self.store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
+                child_name = store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
                 log_info("created child", child_name)
 
                 cmd_parts = self.hparam_search.apply_runset(arg_dict, context.cmd_parts, context, store, child_name)
@@ -812,7 +862,7 @@ class XTController(rpyc.Service):
 
             hp_set = str(arg_dict)
         else:
-            child_name = self.store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
+            child_name = store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
             log_info("created child", child_name)
 
             if self.runsets:
@@ -862,6 +912,7 @@ class XTController(rpyc.Service):
             context.node_index = self.node_index
 
             # log run CMD
+            store = self.get_store()
             store.log_run_event(context.ws, child_name, "cmd", {"cmd": cmd_parts, "xt_cmd": context.xt_cmd}, job_id=context.job_id)  
 
             # for now, don't log context (contain private credentials and not clear if we really need it)
@@ -870,7 +921,7 @@ class XTController(rpyc.Service):
 
             child_info = RunInfo(child_name, context.ws, cmd_parts, context.run_script, 
                 None, context, "running", True, parent_name=parent.run_name, mirror_close_func = self.stop_mirror_worker, 
-                node_id=self.node_id, run_index=run_index, store=self.store, is_parent=False)
+                node_id=self.node_id, run_index=run_index, store=store, is_parent=False)
 
             parent.process_run_output("spawned: {}\n".format(child_name))
         else:
@@ -887,6 +938,7 @@ class XTController(rpyc.Service):
         ''' be conservative here - don't assume we have even started the process.
         '''
         log_info("exit_handler", run_info.run_name)
+        store = self.get_store()
 
         if not run_info.process_was_created:
             # run died during launch (likely due to Azure/dbDB errors)
@@ -912,7 +964,8 @@ class XTController(rpyc.Service):
             # log run end to JOB
             dd = {"node_id": self.node_id, "run": run_info.run_name, "status": run_info.status, 
                 "is_parent": run_info.is_parent, "exit_code": run_info.exit_code}
-            self.store.log_node_event(self.ws_name, self.job_id, self.node_id, "end_run", dd)
+
+            store.log_node_event(self.ws_name, self.job_id, self.node_id, "end_run", dd)
 
             log_info("run_end logged", run_info.run_name)
 
@@ -942,7 +995,7 @@ class XTController(rpyc.Service):
         context = run_info.context
 
         if run_info.parent_name and context.search_style == "dynamic":
-            stop_controller = self.hparam_search.process_end_of_run(self.store, context)
+            stop_controller = self.hparam_search.process_end_of_run(store, context)
 
             if stop_controller:
                 self.stop_processing_runs = True
@@ -1127,7 +1180,9 @@ class XTController(rpyc.Service):
 
         run_name = context.run_name
         exper_name = context.exper_name
+
         log_title("QUEUING run: {}".format(run_name))
+        store = self.get_store()
 
         #working_dir = os.path.expanduser(context.working_dir)
         app_name = context.app_name
@@ -1152,10 +1207,9 @@ class XTController(rpyc.Service):
 
         run_info = RunInfo(run_name, context.ws, cmd_parts, run_script, context.repeat, context, "queued", True, 
             parent_name=None, parent_prep_needed=parent_prep_needed, mirror_close_func = self.stop_mirror_worker, 
-            node_id=self.node_id, run_index=None, store=self.store, max_delay=context.max_delay, is_parent=is_parent)
+            node_id=self.node_id, run_index=None, store=store, max_delay=context.max_delay, is_parent=is_parent)
 
         # log run QUEUED event 
-        store = self.store
         store.log_run_event(context.ws, run_name, "queued", {}, job_id=context.job_id) 
 
         # queue job to be run
@@ -1202,7 +1256,7 @@ class XTController(rpyc.Service):
 
         self.start_local_run_core(run_info, cmd_parts, rundir, run_dir_index)
 
-    def start_local_run_core(self, run_info, cmd_parts, rundir, rundir_index):
+    def start_local_run_core(self, child_ri:RunInfo, cmd_parts: list, rundir: str, rundir_index: int):
         '''
         Note: 
             when user did NOT specify a run script:
@@ -1217,11 +1271,11 @@ class XTController(rpyc.Service):
         '''
         log_info("cmd_parts", cmd_parts)
 
-        context = run_info.context  
-        run_name = run_info.run_name
+        context = child_ri.context  
+        run_name = child_ri.run_name
 
         # download files from STORE to rundir
-        store = self.store
+        store = self.get_store()
         job_id = context.job_id 
 
         if self.is_aml:
@@ -1265,7 +1319,7 @@ class XTController(rpyc.Service):
         files = capture.download_before_files(store, context.job_id, context.ws, run_name, dest_dir=rundir, silent=True, source="run")
         log_info("downloaded", "{} BEFORE files from RUN STORE".format(len(files)))
 
-        run_script = run_info.run_script
+        run_script = child_ri.run_script
         script_args = None
 
         if run_script:
@@ -1311,7 +1365,7 @@ class XTController(rpyc.Service):
         # update XT_OUTPUT_DIR and XT_OUTPUT_MNT for child run path
         output_path = os.getenv("XT_OUTPUT_DIR")
         log_info("parent output path", output_path)
-        parent_name = run_info.parent_name
+        parent_name = child_ri.parent_name
         log_info("parent name", parent_name)
 
         if output_path and parent_name:
@@ -1332,7 +1386,7 @@ class XTController(rpyc.Service):
         log_info("run_script", run_script)
 
         # this expands symbols in the script AND removes CR chars for linux scripts
-        run_script = scriptor.fixup_script(run_script, pc_utils.is_windows(), True, run_info=run_info, concurrent=self.concurrent)  
+        run_script = scriptor.fixup_script(run_script, pc_utils.is_windows(), True, run_info=child_ri, concurrent=self.concurrent)  
 
         # write RUN SCRIPT LINES to a run_appXXX script file
         if pc_utils.is_windows():
@@ -1348,8 +1402,8 @@ class XTController(rpyc.Service):
             scriptor.write_script_file(run_script, fn_script, for_windows=False)
             log_info("LINUX script", fn_script)
 
-        console_fn = rundir + "/output/console.txt"
-        run_info.set_console_fn(console_fn)
+        console_fn = rundir + "/service_logs/stdboth.txt"
+        child_ri.set_console_fn(console_fn)
 
         log_info("target", child_env["XT_TARGET_FILE"])
 
@@ -1383,11 +1437,8 @@ class XTController(rpyc.Service):
         store.log_node_event(self.ws_name, self.job_id, self.node_id, "start_run", dd)
 
         # start a MIRROR thread to copy files to grok server
-        if context.mirror_dest and context.mirror_files and (context.grok_server or context.mirror_dest == "storage"):
-            log_info("mirror_dest=", context.mirror_dest)
-            log_info("mirror_files=", context.mirror_files)
-
-            self.start_mirror_worker(store, run_info, rundir, run_name, context)
+        if context.mirror_request_list:
+            self.start_mirror_workers_for_run(store, child_ri, rundir, run_name, context)
 
         # tell db JOBS that this job has a new run
         db = store.get_database()
@@ -1410,12 +1461,12 @@ class XTController(rpyc.Service):
         store.log_run_event(context.ws, run_name, start_event_name, {}, job_id=job_id)  
         #prep_script = run_info.prep_script  
 
-        if not job_id in self.running_jobs:
+        if not self.running_job:
             # this is the first run of this job on this node
-            self.running_jobs[job_id] = True
+            self.running_job = job_id
 
         # don't start run until above logging has completed
-        self.start_run_now(rundir, parts, cwd, fn_script, child_env, stdout_is_text, bufsize, run_info)
+        self.start_run_now(rundir, parts, cwd, fn_script, child_env, stdout_is_text, bufsize, child_ri)
 
         return True
 
@@ -1453,36 +1504,73 @@ class XTController(rpyc.Service):
 
         # start a thread to consume STDOUT and STDERR from process
         thread_name = "watcher_" + run_info.run_name
-        stdout_thread = Thread(name=thread_name, target=read_from_pipe, args=(process.stdout, 
+        stdout_thread = Thread(name=thread_name, target=read_from_stdout_pipe, args=(process.stdout, 
             run_info, self, stdout_is_text))
         stdout_thread.start()
 
         log_info("process created", p)
 
-    def start_mirror_worker(self, store, run_info, rundir, run_name, context):
+    def start_mirror_workers_for_run(self, store, run_info, rundir, run_name, context):
         log_info("starting MIRROR worker for run", run_name)
-        log_info("mirror-dest", context.mirror_dest)
-        log_info("mirror-path", context.mirror_files)
 
-        worker = MirrorWorker(store, rundir, context.mirror_dest, context.mirror_files, context.grok_server, context.ws, run_name, 
-            job_id=context.job_id, mirror_delay_mins=context.mirror_delay_mins, 
-            show_calls=context.show_mirror_calls)
-        
-        run_info.mirror_worker = worker
-        self.mirror_workers.append(worker)
+        mirror_requests = context.mirror_request_list
+        mirror_dest: str = context.mirror_dest
 
-        worker.start()
+        log_info("RUN mirror-dest", mirror_dest)
+        log_info("RUN mirror_requests=", mirror_requests)
+
+        workers = []
+
+        for mr in mirror_requests:
+            dest_type = mr["dest_type"]
+
+            if dest_type == "run":
+
+                worker = MirrorWorker(store, rundir, mirror_dest, wildcard_path=mr["local_dir"], grok_url=context.grok_server, 
+                    ws_name=context.ws, run_name=run_name, node_id=None, job_id=context.job_id, mirror_delay_mins=context.mirror_delay_mins, 
+                    show_calls=context.show_mirror_calls, dest_folder=mr["dest_folder"])
+            
+                worker.start()
+                workers.append(worker)
+
+                self.mirror_workers.append(worker)
+
+        run_info.mirror_workers = workers
+
+    def start_mirror_workers_for_node(self, store,  context):
+
+        mirror_requests = context.mirror_request_list
+        mirror_dest: str = context.mirror_dest
+
+        log_info("NODE mirror-dest", mirror_dest)
+        log_info("NODE mirror_requests=", mirror_requests)
+
+        for mr in mirror_requests:
+            dest_type = mr["dest_type"]
+
+            if dest_type == "node":
+                node_id = utils.node_id(context.node_index)
+                cwd = os.getcwd()
+
+                worker = MirrorWorker(store, cwd, mirror_dest, wildcard_path=mr["local_dir"], grok_url=context.grok_server, 
+                    ws_name=context.ws, run_name=None, node_id=node_id, job_id=context.job_id, mirror_delay_mins=context.mirror_delay_mins, 
+                    show_calls=context.show_mirror_calls, dest_folder=mr["dest_folder"])
+            
+                worker.start()
+
+                self.mirror_workers.append(worker)
 
     def stop_mirror_worker(self, run_info):
-        log_info("stop_mirror_worker: run_info.mirror_worker", run_info.mirror_worker)
+        log_info("stop_mirror_worker: run_info.mirror_workers", run_info.mirror_workers)
         
-        if run_info.mirror_worker:
-            worker = run_info.mirror_worker
-            worker.stop()
+        if run_info.mirror_workers:
+            for worker in run_info.mirror_workers:
+                worker.stop()
 
-            if worker in self.mirror_workers:
-                self.mirror_workers.remove(worker)
-            run_info.worker = None
+                if worker in self.mirror_workers:
+                    self.mirror_workers.remove(worker)
+            
+            run_info.mirror_workers = None
 
     def diag(self, msg):
         console.print(msg)
@@ -1567,29 +1655,6 @@ class XTController(rpyc.Service):
                     result += run_info.get_summary_stats() + "\n"
         return result
 
-    # def cancel_core(self, run_info, for_restart=False):
-    #     result = None
-    #     status = None
-        
-    #     log_info("----> killing", "{}/{}".format(run_info.workspace, run_info.run_name))
-
-    #     with queue_lock:
-    #         if run_info in self.queue:
-    #             self.queue.remove(run_info)
-
-    #     # log run KILLED event 
-    #     context = run_info.context
-    #     store = self.store
-
-    #     if not for_restart:
-    #         store.log_run_event(context.ws, run_info.run_name, "cancelled", {})    
-
-    #     with run_info.lock:
-    #         run_info.killed_for_restart = for_restart
-    #         result, status, before_status = run_info.kill()
-
-    #     return result, status, before_status
-
     def get_matching_run_infos(self, full_run_names):
         # match all runinfos that have not finished (exact match and matching children)
         matches = []
