@@ -36,7 +36,7 @@ def get_command_dicts(prev_exper, curr_exper, browse_flag, browse_opt, timeout_o
 
 	    # upload DATA/MODELS (depends on miniMnist generated data and model)
 	    {"title": "upload MNIST dataset", "cmd": "xt upload ./data/MNIST/processed/** MNIST/processed --share=data"},
-	    {"title": "upload previously trained MNIST model", "cmd": "xt upload .\models\miniMnist\** miniMnist --share=models"},
+	    {"title": "upload previously trained MNIST model", "cmd": "xt upload ./models/miniMnist/** miniMnist --share=models"},
 
 	    # RUNS 1-4
 	    {"title": "run script on LOCAL MACHINE", "cmd": "xt run {}--target=local --exper={} code\miniMnist.py".format(monitor_opt, curr_exper)},
