@@ -44,7 +44,7 @@ class DGDSearch(implements(HpSearchInterface)):
         '''
         use dgd module to perform dgd search for next hparam set.
         '''
-        dgd = DGD()
+        dgd = DGD(context.num_dgd_seeds)
         dgd.build(store, hp_records, runs, context.primary_metric, maximize_metric=context.maximize_metric)
         
         runset = dgd.choose_config()
@@ -159,9 +159,10 @@ class Hyperparameter(object):
 
 
 class DGD(object):
-    def __init__(self):
+    def __init__(self, num_dgd_seeds=100):
         self.runsets = []
         self.configstr_runset_dict = {}
+        self.num_dgd_seeds = num_dgd_seeds
 
     def build(self, store, records, all_runs, primary_metric, report=True, maximize_metric=True):
         self.store = store
@@ -268,8 +269,8 @@ class DGD(object):
         console.print("{} max runs per runset".format(self.max_runs_per_runset))
 
     def choose_config(self):
-        # If there are no runs yet, just return a random configuration.
-        if len(self.runsets) == 0:
+        # If there are not enough completed runs, just return a random configuration.
+        if len(self.runsets) < self.num_dgd_seeds:
             hp_id__setting_id__list = []
             for hparam in self.hparams:
                 last_setting_id = len(hparam.settings) - 1
@@ -369,6 +370,7 @@ def fake_call_from_job(job_name, workspace, fn_hp_search):
     context.providers = config.get("providers")
     context.aggregate_dest = "job"
     context.dest_name = job_name
+    context.num_dgd_seeds = 103
     context.ws = workspace
 
     from xtlib.hparams.hparam_search import HParamSearch
