@@ -17,6 +17,7 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import constants
 from xtlib import run_helper
+from xtlib import store_utils
 from xtlib.console import console
 from xtlib.hparams import hp_helper
 
@@ -26,6 +27,8 @@ class HParamSearch():
         self.is_aml = is_aml
         self.run_history = []
         self.end_id = 0
+        self.search_provider = None
+        self.hp_records = None
 
     # main ENTRY POINT 
     def process_child_hparams(self, child_name, store, context, parent):
@@ -33,8 +36,7 @@ class HParamSearch():
         determine set of hyperparameter values for child_name, update config file, 
         and return updated cmd_parts, as appropriate for search type
         '''
-        cmd_parts = list(context.cmd_parts)
-        sweep_text = None
+        arg_dict = None
 
         if context.hp_config:
             # read hp config file from experiment or job
@@ -44,11 +46,9 @@ class HParamSearch():
                 # assume sweeps file is at the job level
                 text = store.read_job_file(context.ws, context.dest_name, context.hp_config)
 
-            arg_dict = self.generate_hparam_set(context.hp_config, text, child_name, cmd_parts, store, context)
-
-            cmd_parts = self.apply_runset(arg_dict, cmd_parts, context, store, child_name)
-
-        return cmd_parts
+            arg_dict = self.generate_hparam_set(text, child_name, store, context)
+            
+        return arg_dict
 
     def apply_runset(self, arg_dict, cmd_parts, context, store, child_name):
         '''
@@ -80,7 +80,7 @@ class HParamSearch():
 
         return cmd_parts
 
-    def generate_hparam_set(self, fn_config, search_file_text, run_name, cmd_parts, store, context):
+    def generate_hparam_set(self, search_file_text, run_name, store, context):
         '''
         generate a set of hyperparameter values, based on the algorithm specified by
         context.search_type.
@@ -109,25 +109,65 @@ class HParamSearch():
         need_runs = impl.need_runs()
         runs = None
 
+        utils.log_title("HP_SEARCH_CORE")
+
+        utils.log_info("run_name", context.run_name)
+        utils.log_info("primary_metric", context.primary_metric)
+        utils.log_info("maximize_metric", context.maximize_metric)
+
         if need_runs:
             # this could be a lengthly delay
             started = time.time()
             runs = self.get_completed_runs(store, context) 
             elapsed = time.time() - started
-            utils.log_info("hp_search_core, runs", len(runs))
-            utils.log_info("hp_search_core, new", self.new_run_count)
-            utils.log_info("hp_search_core, elapsed", elapsed)
 
+            utils.log_info("get_completed_runs, elapsed", elapsed)
+            utils.log_info("len(all_runs)", len(runs))
+            utils.log_info("new_run_count", self.new_run_count)
+
+        utils.log_title("HP SEARCH: " + search_type)
+
+        # workaround issue for quicktest with older runs using 'seed' hparam
+        if runs:
+            for run in runs:
+                if "hparams" in run:
+                    hparams = run["hparams"]
+                    if "seed" in hparams:
+                        del hparams["seed"]
+            
         arg_dict = impl.search(run_name, store, context, space_records, runs=runs)
 
         # fix up returned values 
-        for key, value in arg_dict.items():
-            if isinstance(value, np.float64):
-                arg_dict[key] = value.item()
+        if arg_dict:
+            for key, value in arg_dict.items():
+                if isinstance(value, (np.float32, np.float64)):
+                    arg_dict[key] = value.item()
+
+            console.print("\n---- search_type={}, returned arg_dict={} ----\n".format(search_type, arg_dict))
+
+        # save for processing end of run
+        self.search_provider = impl
+        self.hp_records = space_records
 
-        console.print("\n---- search_type={}, returned arg_dict={} ----\n".format(search_type, arg_dict))
         return arg_dict
 
+    def make_values_consistent(self, values):
+        '''
+        To try and prevent errors in hyperopt (bayesian search), if any values 
+        in list are float, make them all float.
+        '''
+        is_float = False
+
+        for value in values:
+            if isinstance(value, float):
+                is_float = True
+                break
+
+        if is_float:
+            values = [float(val) for val in values]
+
+        return values
+
     def parse_hp_config_yaml(self, hparams, search_type):
         # use original size=() for randint on hyperopt (bayesian) search
         hp_size = (search_type == "bayesian")
@@ -138,18 +178,28 @@ class HParamSearch():
             fa = hp_helper.parse_hp_dist(value)
             space_func = hp_helper.build_dist_func_instance(prop, fa["func"], fa["args"], hp_size)
 
-            # for DGD: generate a text-file compatible version  of value
-            if isinstance(value, str) and value.startswith("$linspace"):
+            # for DGD: generate a text-file compatible version of value
+            if isinstance(value, str) and value.startswith("$randint"):
+                # looks like "randint" is no longer processed by DGD search, so just pick a value now
+                import random
+                vs = str(random.randint(0, 65535))
+
+            elif isinstance(value, str) and value.startswith("$linspace"):
                 # the values generated from linspace are in "args"
                 args = list(fa["args"])
                 vs = str(args)
-            elif isinstance(value, (list,tuple)):  #  and isinstance(value[0], str):
+
+            elif isinstance(value, (list,tuple)):  
+                # this is the most commonly used XT search space: a list of values
+                value = self.make_values_consistent(value)
+
                 strs = [str(v) for v in value]
                 vs = (", ").join(strs)
+
             else:
                 vs = str(value).strip()
 
-            if vs.startswith("["):
+            if isinstance(vs, str) and vs.startswith("["):
                 vs = vs[1:-1]
                 
             record = {"name": prop, "value": vs, "space_func": space_func}
@@ -188,8 +238,22 @@ class HParamSearch():
         else:
             errors.general_error("unrecognized aggregation dest: {}".format(context.aggregate_dest))
             
-        # we are using in-memory cache (self.run_history)
-        filter_dict["end_id"] = {"$gt": self.end_id}
+        # get next "end_id" to be assigned to an ending run
+        next_end_id = store.database.get_next_end_id_without_update(context.ws)
+
+        # we are using in-memory cache (self.run_history holds runs whose end_id is <= self.end_id)
+        # we specify a max value for end_id so queries return consistent results (avoid dup records)
+        # build a filter to find all runs where: end_id > self.end_id and end_id < next_end_id
+        #filter_dict["end_id"] = {"$gt": self.end_id}
+
+        # TODO: improve this
+        # it is not ideal to have separate code here for each storge version, but v3 is not
+        # yet equiped to handle the mongo implicit $and format, and this form of the "$and" operator
+        # (as a field name value) is not supported by mongo (v1 format).
+        if store_utils.STORAGE_FORMAT == "1":
+            filter_dict["end_id"] = {"$gt": self.end_id}     # , "$lt": next_end_id} 
+        else:
+            filter_dict["end_id"] = {"$and": [{"$gt": self.end_id}, {"$lt": next_end_id} ] }
 
         # only look at runs whose status = "complete"
         filter_dict["status"] = "completed"
@@ -208,11 +272,16 @@ class HParamSearch():
             use_cache=False)
 
         # merge new runs with previous
-        utils.log_info("get_completed_runs, run_history", len(self.run_history))
-        utils.log_info("get_completed_runs, new_runs", len(new_runs))
+        # utils.log_info("filter_dict", filter_dict)
+        # utils.log_info("fields_dict", fields_dict)
+        
+        # utils.log_info("get_completed_runs, self.end_id", self.end_id)
+        # utils.log_info("get_completed_runs, next_end_id", next_end_id)
+        # utils.log_info("get_completed_runs, run_history", len(self.run_history))
+        # utils.log_info("get_completed_runs, new_runs", len(new_runs))
 
         runs = self.run_history + new_runs
-        
+
         # update cache
         self.run_history = runs
         self.new_run_count = len(new_runs)
@@ -222,6 +291,14 @@ class HParamSearch():
             self.end_id = max(end_ids)
 
         return runs
+
+    def process_end_of_run(self, store, context):
+        utils.log_title("HP SEARCH (PROCESS_END_OF_RUN): " + context.search_type)
+        utils.log_info("run_name", context.run_name)
+        utils.log_info("primary_metric", context.primary_metric)
+
+        self.search_provider.process_end_of_run(store, context, self.hp_records)
+        console.print("")
    
     def apply_runset_to_cmd_parts(self, arg_dict, cmd_parts, context, insert_before=False):
         # if insert_before=True, we would like to insert the new HP values BEFORE the potential override options
