@@ -10,7 +10,7 @@ from textwrap import wrap
 import time
 import datetime
 import tempfile
-from xtlib import report_builder
+from xtlib import node_helper, report_builder
 import numpy as np 
 
 from xtlib.client import Client
@@ -602,13 +602,13 @@ class ImplCompute(ImplBase):
     def get_runs_by_box(self, run_names, workspace=None):
         run_names, actual_ws = run_helper.parse_run_list(self.store, workspace, run_names)
 
-        mongo = self.store.get_database()
-        fields_dict = {"box_name": 1, "compute": 1, "ws_name": 1, "run_name": 1}
+        db = self.store.get_database()
+        fields_dict = {"box_name": 1, "compute": 1, "ws_name": 1, "run_name": 1, "job_id": 1}
 
         filter_dict = {}
         filter_dict["run_name"] = {"$in": run_names}
 
-        box_records = mongo.get_info_for_runs(actual_ws, filter_dict, fields_dict)
+        box_records = db.get_info_for_runs(actual_ws, filter_dict, fields_dict)
 
         # group by box_name
         runs_by_box = {}
@@ -636,8 +636,6 @@ class ImplCompute(ImplBase):
         for box_name, runs in runs_by_box.items():
             first_run = runs[0]
 
-            ws_name = workspace    # first_run["ws_name"]
-            # = [ws_name + "/" + run["run_name"] for run in runs]
             run_names = [run["run_name"] for run in runs]
             compute = first_run["compute"] if "compute" in first_run else "pool"    # some legacy support
 
@@ -646,6 +644,18 @@ class ImplCompute(ImplBase):
 
             cancel_results_by_box[box_name] = cancel_results
 
+            if "_" in box_name:
+                # normal backend service
+                job_id, service_id, node_index = box_name.split("-")
+
+            else:
+                # single box name
+                job_id = first_run["job_id"]
+                node_index = 0
+                service_id = "pool"
+
+            self.store.database.mark_node_cancelled(workspace, job_id, node_index)
+
         self.print_cancel_all_results(cancel_results_by_box)
 
     #---- CANCEL JOB command ----
@@ -653,7 +663,7 @@ class ImplCompute(ImplBase):
     @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
     @option("max-workers", type=int, default="$general.max-run-workers", help="the max number of background workers for this command")
     @flag("cancel", default=1, help="call the backend service to cancel the job")
-    @flag("wrapup", default=1, help="wrapup the runs associated with the job")
+    @flag("wrapup", default=1, help="wrapup the nodes and runs associated with the job")
     @example(task="cancel jobs job3 and job15", text="xt cancel job job3, job15")
     @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels the specified jobs and their active or queued runs")
     def cancel_job(self, job_names, workspace, max_workers, cancel, wrapup):
@@ -676,10 +686,17 @@ class ImplCompute(ImplBase):
                 # wrap up runs associated with each node
 
             if wrapup:
-                # whether or not the cancel succeeded, cleanup the runs
+                # whether or not the cancel succeeded, cleanup the RUNS
                 node_list = list(service_info_by_node.keys())
                 runs = run_helper.wrapup_runs_for_nodes(self.store, workspace, job_id, node_list, max_workers)
 
+                # cleanup the NODES
+                for node_index, node in enumerate(node_list):
+                    self.store.database.mark_node_cancelled(workspace, job_id, node_index)
+
+                # cleanup the JOB
+                self.store.database.mark_job_cancelled(workspace, job_id)
+
     # TODO: this will replace the 'view status' command
     #---- LIST QUEUES command ----
     @argument("target", help="the name of the compute target")
@@ -917,7 +934,8 @@ class ImplCompute(ImplBase):
     @option("hp-config", default="$hyperparameter-search.hp-config", type=str, help="the path of the hyperparameter config file")
     @flag("jupyter-monitor", help="when True, a Jupyter notebook is created to monitor the run")
     @hidden("log", default="$logging.log", help="specifes if run-related events should be logged")
-    @hidden("log-reports", default="$logging.log-reports", help="specifies if CPU, GPU, and MEMORY reports should be included in setup log")
+    @flag("log-reports", default="$after-files.log-file-uploads", help="if set, log each AFTER file/size before uploading")
+    @hidden("log-file-uploads", default="$logging.log-reports", help="specifies if CPU, GPU, and MEMORY reports should be included in setup log")
     @option("low-pri", type=bool, default=DEFAULT_FROM_COMPUTE, help="when true, use low-priority (preemptable) nodes for this job")
     @option("max-passes", default="$hyperparameter-search.max-passes", type=int, help="used to limit number of passes over hyperparameters in hp search")
     @option("max-minutes", default="$hyperparameter-search.max-minutes", type=int, help="the maximum number of minutes the run can execute before being terminated")
@@ -929,6 +947,7 @@ class ImplCompute(ImplBase):
     @hidden("merge-batch-logs", type=int, default="$logging.merge-batch-logs", help="replaces STDOUT.txt and STDERR.txt with STDBOTH.txt")
     @hidden("mirror-dest", default="$logging.mirror-dest", help="the location where mirrored information is stored (none, storage, grok)")
     @hidden("mirror-files", default="$logging.mirror-files", help="the wildcard path that specifies files to be mirrored")
+    @hidden("mirror-log-files", default="$logging.mirror-log-files", help="when true, the job's log file directory is mirrored")
     @option("model-action", default="$model.model-action", help="the model action to take on the target, before run is started")
     @hidden("model-local", default="$model.model-local", help="the path on the local machine specifying where the model for this job resides")
     @hidden("model-share-path", default="$model.model-share-path", help="the model share name for the model")
@@ -947,6 +966,7 @@ class ImplCompute(ImplBase):
     @option("parent-script", type=str, help="path of script used to initialize the target for repeated runs of primary script")
     @hidden("pip-freeze", default="$logging.pip-freeze", help="when true, installed pip packges are included in the setup log")
     @hidden("primary-metric", default="$general.primary-metric", help="the name of the metric to use for hyperparameter searching")
+    @option("python-version", type=str, help="the python version passed to AML/ITP/Singularity for pip and conda operations")
     @option("queue", type=str, help="the name of the Philly queue to use when submitting this job")
     @hidden("remote-control", default="$general.remote-control", help="specifies if XT controller will listen for XT client commands")
     @hidden("report-rollup", default="$run-reports.report-rollup", help="whether to rollup metrics by primary metric or just use last reported metric set")
