@@ -6,8 +6,6 @@
 import os
 import re
 import sys
-import copy
-import time
 import shutil
 import psutil
 import logging
@@ -29,6 +27,7 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import pc_utils
 from xtlib import constants
+from xtlib import job_helper
 from xtlib import run_helper
 from xtlib import file_utils
 from .backends import backend_aml 
@@ -194,28 +193,54 @@ class ImplUtilities(ImplBase):
         console.print("downloaded to: {}".format(fn_output))
 
     #---- SSH command ----
-    @argument(name="name", help="the box or run name to communicate with or connect to")
+    @argument(name="name", help="the name of the job, run, or box to connect with")
     @argument(name="cmd", help="the optional command to execute", required=False, type="text", default="")
     @option(name="workspace", default="$general.workspace", help="the workspace for the runs to be displayed")
     @option(name="output", help="the name of the file to write the cmd output to")
+    @option(name="node-index", type=int, default=0, help="the node index for the specified job")
     @example(task="initiate a remote console session with box 'vm10'", text="xt ssh vm10")
     @example(task="get a directory listing of files on box 'vm23''", text="xt ssh vm10 ls -l")
     @command(options_before_args=True, help="executes the specified command, on begins a console session, with the specified box")
-    def ssh(self, name, cmd, workspace, output):
-
+    def ssh(self, name, cmd, workspace, output, node_index):
+        # TODO: fix issue where XT is putting all options into "cmd"
         capture_output = True if cmd else False
 
         if run_helper.is_run_name(name):
-            # assume it's a RUN name
-            rr = run_helper.get_run_record(self.store, workspace, name)
-            if not "cluster" in rr:
-                errors.store_error("only philly runs are currently supported for this cmd")
+            job_id, node_index = run_helper.get_job_node_index(self.store, self.core, workspace, name)
+        elif job_helper.is_job_id(name):
+            job_id = name
+        else:
+            job_id = None
+
+        if job_id:
+            import clipboard
+
+            # we have a job_id and a node_index (that uniquely identifies the compute node)
+            si, backend = job_helper.get_service_node_info_with_backend(self.store, self.core, workspace, job_id, node_index)
+            if backend.get_name() != "batch":
+                errors.store_error("only Azure Batch jobs/runs are currently supported for this cmd")
+
+            cd = backend.get_ssh_creds(self.config, job_id, node_index, workspace)
+
+            # copy password to clipboard
+            pw = cd["pw"]
+            clipboard.copy(pw)
+
+            print("")
+            print("running SSH:")
+            print("  - the SSH password has been copied to the clipboard")
+
+            if not cmd:
+                print("  - to attach to a running docker container: ")
+                print("      $ sudo docker ps")
+                print("      $ sudo docker exec -it <container id> /bin/bash")
+                print("      $ cd ~/.xt")
+                print("")
+
+            ssh_cmd = "ssh {}@{} -p{} {}".format(cd["user"], cd["ip_addr"], cd["port"], cmd)
+            print(ssh_cmd)
+            os.system(ssh_cmd)
 
-            philly = Philly(core=self.core)
-            ssh_cmd = philly.get_ssh_for_run(workspace, name)
-            print("ssh_cmd: " + ssh_cmd)
-            
-            exit_code, output = process_utils.sync_run(ssh_cmd, report_error=True, capture_output=capture_output)
         else:
             # assume it's a BOX name
             info = box_information.get_box_addr(self.config, name, self.store)
@@ -227,12 +252,12 @@ class ImplUtilities(ImplBase):
             exit_code, ssh_output = process_utils.sync_run_ssh(self, ssh_ip, cmd, capture_output=capture_output,
                 capture_as_bytes=capture_as_bytes)
 
-        if output:
-            # write as bytes
-            with open(output, "wb") as outfile:
-                outfile.write(ssh_output)
-        elif capture_output:
-            console.print(ssh_output)
+            if output:
+                # write as bytes
+                with open(output, "wb") as outfile:
+                    outfile.write(ssh_output)
+            elif capture_output:
+                console.print(ssh_output)
 
     #---- SCP command ----
     @argument(name="cmd", help="the scp command to execute", type="text")
@@ -636,6 +661,7 @@ class ImplUtilities(ImplBase):
                 fn_archive = os.path.join(file_utils.get_xtlib_dir(), "demo_files", "xt-demo-archive.zip")
                 impl_storage_api.import_workspace(fn_archive, "xt-demo", "xtd", overwrite=overwrite, show_output=False)
 
+            console.print("to run the demo, move to the demo directory and run 'python xt_demo.py'")
     def is_aml_ws(self, ws_name):
         return self.azure_ml.does_ws_exist(ws_name)
         
