@@ -15,10 +15,28 @@ from xtlib import job_helper
 from xtlib import run_helper
 from xtlib import store_utils
 
+'''
+To add a new node info column (defined at job submit time):
+    - add it to the following node_helper.py dicts:
+        - add to "node_info_props" dict
+        - add to "user_to_actual" dict (recommend using same name for friendly name)
+        - add to "std_cols_desc" dict (describes what the column holds)
+
+    - add it to the following odbc.py dicts:
+        - the node_info sub-dict in _init_core_tables()
+
+    - add it to runner.py:
+        - TBD....
+'''
+
+# NEW COLS: aml_compute, compute_target
+# LEGACY column for old jobs: "target" (aml_compute)
 node_info_props = \
     {
         "_id": 1, 'ws_name': 1, "job_id": 1, "node_name": 1, "node_id": 1, 
-        "node_index": 1, "node_num": 1, "target": 1, "total_runs": 1, 
+        "node_index": 1, "node_num": 1, 
+        "target": 1, "aml_compute": 1, "compute_target": 1,
+        "total_runs": 1, 
         "box_name": 1, "run_name": 1, "exper_name": 1,
         "controller_port": 1, "ip_addr": 1, "secret": 1, "service_info": 1,
     }
@@ -46,6 +64,8 @@ def get_node_property_dicts():
         "node_num": "node_num",
         "exper_name": "exper_name",
         "target": "target",
+        "aml_compute": "aml_compute",
+        "compute_target": "compute_target",
         "total_runs": "total_runs",
         "box_name": "box_name",
         "run_name": "run_name",
@@ -86,7 +106,9 @@ def get_node_property_dicts():
         "node_index": "the index of the node, within the job",
         "node_num": "a combination of the job/node that is used to sort by the node name",
         "exper_name": "experiment name associated with this node's job",
-        "target": "the name of the compute target this node is part of",
+        "target": "the name of the AML compute for this node (legacy column)",
+        "aml_compute": "the name of the AML compute for this node",
+        "compute_target": "the name of the compute target for this node",
         "total_runs": "the number of runs assigned to this node",
         "box_name": "the user-friendly name of this node",
         "run_name": "the name of the flat or parent run associated with this node",
