@@ -250,6 +250,13 @@ class PlotBuilder():
             #     continue
 
             metric_sets = self.extract_data_from_run(record, no_metrics, run)
+
+            if not self.x_col:
+                # if "step" col has not yet been assigned, use first data to set step col name
+                ms = metric_sets[0]
+                step_name = ms["records"][0]["__step_name__"]
+                if step_name:
+                    self.x_col = step_name
             
             missing_col_runs = self.build_data_frames_from_metric_sets(run, metric_sets, missing_metrics, 
                 node, job, experiment, workspace, data_frames_by_cols, record)
