@@ -16,13 +16,14 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import run_helper
 from xtlib import job_helper
-from xtlib import qfe
 
 '''
 Definitions:
     argument - a postional, unnamed parameter; can be optional iif it last argument
     option   - a non-positional, named parameter; can be optional or required
     flag     - a non-positional, named parameter without a value, or with a 1, 0 value
+    layer    - marker to start a new layer of arguments for the current command
+    unknown  - option that accepts unknown options and passes them to command
 
 Root:
     options can be added to the main() function with a "root=True" parameter.  This
@@ -45,7 +46,7 @@ current_cmd_info = None         # for command function being defined as function
 parser_cmd_info = None          # the command being parsed or dispatched
 command_help_func = None
 kwgroup_help_func = None
-explict_options = {}
+explicit_options = {}
 
 # debugging flags
 debug_decorators = False
@@ -369,6 +370,50 @@ def flag(name, default=None, help=""):
 
     return decorator_flag
 
+# LAYER decorator processor  
+def layer(name, dest, help=""):
+    def decorator_layer(func):
+        @functools.wraps(func)
+        def wrapper_layer(*args, **kwargs):
+            return func(*args, **kwargs)
+
+        if debug_decorators:
+            console.print("layer decorator called, name=", name, ", func=", func.__name__)
+
+        global current_cmd_info, root_cmd_info
+
+        if not current_cmd_info:
+            errors.internal_error("@layer decorators must be followed by a single @command or @root decorator")
+
+        option_info = {"name": name, "hidden": False, "type": "layer", "multiple": False, "help": help, "dest": dest}
+        update_or_insert_argument(current_cmd_info, "options", option_info)
+
+        return wrapper_layer
+
+    return decorator_layer
+
+# UNKNOWN decorator processor  
+def unknown(dest, type, help=""):
+    def decorator_unknown(func):
+        @functools.wraps(func)
+        def wrapper_unknown(*args, **kwargs):
+            return func(*args, **kwargs)
+
+        if debug_decorators:
+            console.print("unknown decorator called, name=", name, ", func=", func.__name__)
+
+        global current_cmd_info, root_cmd_info
+
+        if not current_cmd_info:
+            errors.internal_error("@unknown decorators must be followed by a single @command or @root decorator")
+
+        option_info = {"name": "_unknown_", "hidden": False, "type": type, "multiple": False, "help": help, "dest": dest}
+        update_or_insert_argument(current_cmd_info, "options", option_info)
+
+        return wrapper_unknown
+
+    return decorator_unknown
+
 # EXAMPLE decorator processor  
 def example(text, task="", image=None):
     def decorator_example(func):
@@ -476,15 +521,15 @@ def get_explicit_options():
     '''
     return dict of options explicitly set for this command (dash-style names)
     '''
-    return explict_options
+    return explicit_options
 
 def set_explicit_options(options, additive=True):
-    global explict_options
+    global explicit_options
     
     if additive:
-        explict_options.update(options)
+        explicit_options.update(options)
     else:
-        explict_options = options
+        explicit_options = options
 
 class Dispatcher():
     def __init__(self, impl_dict, config, preprocessor=None):
@@ -495,6 +540,10 @@ class Dispatcher():
         self.dispatch_cmd = None
         self.cmd_info = None
         self.user_filters = self.config.data["user-filters"]
+        self.unknown_dict = {}
+        self.unknown_dest = None
+        self.layers = []
+        self.unknown_option = None
 
         global commands_by_name, current_dispatcher, dispatch_counter, commands, save_commands
 
@@ -793,6 +842,15 @@ class Dispatcher():
                     .format(name, ", ".join(match_names)))
 
         found = matches[0] if matches else None
+
+        if not found:
+            # check for presence of @unknown for this command
+           if self.unknown_option:
+                # make this looks like a actual match, but with an 'unknown' flag
+                found = dict(self.unknown_option)
+                found["name"] =  name
+                found["unknown"] = True
+
         return found
 
     def list_item_by_value(self, value, values):
@@ -962,6 +1020,7 @@ class Dispatcher():
         return value
 
     def process_prop_op_value(self, value):
+        
         # mini parse of the value: <prop> <op> <value>
         scanner = Scanner(value)
         prop = scanner.scan(False)
@@ -991,12 +1050,16 @@ class Dispatcher():
             if not found:
                 self.syntax_error("Value for option {} not recognized: {}, must be one of: {}".format(opt_name, value, ", ".join(values)))
             value = found
+
         elif opt_type == "flag":
              value = self.parse_flag_value(opt_name, value)
+
         elif opt_type == "int":
             value = int(value)
+
         elif opt_type == "float":
             value = float(value)
+
         elif opt_type == "bool":
             value = value.lower()
             if value in ["true", "1"]:
@@ -1005,23 +1068,30 @@ class Dispatcher():
                 value = False
             else:
                 self.syntax_error("Illegal value for boolean option: " + str(value))
+
         elif opt_type == "prop_op_value":
             value = self.process_prop_op_value(value)
+
         elif opt_type == "str_list":
             if not isinstance(value, list):
                 value = [value]
+
         elif opt_type == "named_arg_list":
             if not isinstance(value, list):
                 value = [value]
             value = self.convert_str_list_to_arg_dict(value)
+
         elif opt_type == "int_list":
             if not isinstance(value, list):
                 value = [value]
+
         elif opt_type == "num_list":
             if not isinstance(value, list):
                 value = [value]
+
         elif opt_type == "str":
             value = self.expand_system_values(value)
+
         else:
             errors.internal_error("unrecognized option type: {}".format(opt_type))
 
@@ -1103,8 +1173,8 @@ class Dispatcher():
         full_arg_dict = { key.replace("_", "-"):value for key, value in arg_dict.items() }
 
         # remember options that were set explicitly (dash-style)
-        global explict_options
-        explict_options = dict(full_arg_dict)
+        global explicit_options
+        explicit_options = dict(full_arg_dict)
 
         # process all aguments, options, and flags; ensure each has a value in arg_dict
         all_args = arguments + options
@@ -1132,7 +1202,12 @@ class Dispatcher():
 
         # now add default values for all other args
         for info in all_args:
+            # don't add layer marker to args
+            if info["type"] == "layer":
+                continue
+
             name = info["arg_name"] if "arg_name" in info else info["name"]
+            
             required = info["required"] if "required" in info else None
 
             if not name in full_arg_dict:
@@ -1223,12 +1298,22 @@ class Dispatcher():
 
         return template_option, new_args
         
+    def get_layers_dest(self, options):
+        dest = None
+
+        for opt in options:
+            if opt["type"] == "layer":
+                dest = opt["dest"]
+                break
+
+        return dest
+
     def dispatch(self, args, is_rerun=False, capture_output=False, raise_syntax_exception=False):
         self.raise_syntax_exception = raise_syntax_exception
         
         # be sure to reset this for each parse (for multi-command XT sessions)
-        global explict_options
-        explict_options = {}
+        global explicit_options
+        explicit_options = {}
 
         # extract the template option since it has curly brackets that are not converted to quotes
         template_option, args = self.extract_template_option(args)
@@ -1274,6 +1359,14 @@ class Dispatcher():
 
         self.cmd_info = cmd_info
 
+        # does this command use an @unknown option?
+        options = cmd_info["options"]
+        self.unknown_option = None
+        for opt in options:
+            if opt["name"] == "_unknown_":
+                self.unknown_option = opt
+                break
+
         cmd_name = cmd_info["name"]
         self.cmd_words = cmd_name.replace("_", " ")
         func = cmd_info["func"]
@@ -1315,8 +1408,16 @@ class Dispatcher():
         if tok:
             errors.argument_error("end of input", tok)
 
+        # add unknown options, if found
+        if self.unknown_option:
+            self.add_unknown_to_layer(arg_dict, layer_seen=False)
+
         full_arg_dict = self.validate_and_add_defaults(arguments, options, arg_dict)
 
+        dest = self.get_layers_dest(options)
+        if dest: 
+            full_arg_dict[dest] = self.layers
+
         console.diag("dispatching to command func")
 
         # select the caller using function's module name
@@ -1327,6 +1428,9 @@ class Dispatcher():
         if is_rerun:
             full_arg_dict["is_rerun"] = 1
 
+        if "_unknown_" in full_arg_dict:
+            del full_arg_dict["_unknown_"]
+
         # call the matching command function with collected func args
         if self.preprocessor:
             self.preprocessor("command", caller, full_arg_dict)
@@ -1416,6 +1520,38 @@ class Dispatcher():
         arg_dict["filter"].append(filter)
         return tok
 
+    def arguments_contain(self, arguments, name):
+        found = False
+        for arg in arguments:
+            if arg["name"] == name:
+                found = True
+                break
+
+        return found
+
+    def remove_optional_args(self, arg_dict):
+        arguments = self.cmd_info["arguments"]
+
+        for key in list(arg_dict):
+            if not self.arguments_contain(arguments, key):
+                del arg_dict[key]
+
+    def add_unknown_to_layer(self, arg_dict, layer_seen:bool):
+        # this part is hard-coded to get seaborn working with multiple layers
+        # TODO: make use of --type more generalized
+        # add current --type from arg dict
+        if not "type" in arg_dict:
+            if layer_seen:
+                errors.syntax_error("required --type option not specified in previous layer")
+            else:
+                errors.syntax_error("required --type option not specified")
+
+        self.unknown_dict["type"] = arg_dict["type"]
+        self.layers.append(dict(self.unknown_dict))
+
+        del arg_dict["type"]
+        self.unknown_dict = {}
+
     def parse_cmd_option(self, match, scanner, tok, arg_dict):            
 
         tok = scanner.scan()        # skip over name
@@ -1427,14 +1563,30 @@ class Dispatcher():
             found_equals = True
             tok = scanner.scan()        # skip over equals
 
-        values = match["values"] if "values" in match else None
-        #value = self.process_option_value(name, match["type"], value, values)
-        required = utils.safe_value(match, "required")
+        if name == "layer":
+            # layer's job is to accumulate all previous args into a layer dict, and append to self.layers
+            if self.unknown_option:
+                self.add_unknown_to_layer(arg_dict, layer_seen=True)
 
-        value, tok = self.parse_option_value(name, match["type"], values, found_equals, tok, scanner, 
-            required)
+            elif arg_dict:
+                self.layers.append(dict(arg_dict))
+                self.remove_optional_args(arg_dict)
 
-        arg_dict[name] = value
+        else:
+            values = match["values"] if "values" in match else None
+            #value = self.process_option_value(name, match["type"], value, values)
+            required = utils.safe_value(match, "required")
+
+            value, tok = self.parse_option_value(name, match["type"], values, found_equals, tok, scanner, 
+                required)
+
+            if "unknown" in match:
+                # place in special argdict
+                self.unknown_dict[name] = value
+                self.unknown_dest = match["dest"]
+
+            else:
+                arg_dict[name] = value
 
         return tok
 
