@@ -61,24 +61,41 @@ class ReportBuilder():
 
     def get_requested_cols(self, user_col_args, avail_list):
         actual_cols = []
-        new_col_args = {}
+        new_col_args = []
         
-        for key, value in user_col_args.items():
+        for value in user_col_args:
+            key = value["key"]
+
             # if the requested key is available, include it
             if "*" in key:
                 # wildcard match
-                 matches = self.wildcard_matches_in_list(key, avail_list, ["metrics." + constants.STEP_NAME, "metrics.$step_name"])
-                 actual_cols += matches
+                if key.startswith("metrics."):
+                     matches = self.wildcard_matches_in_list(key, name_list=avail_list, omits=["metrics." + constants.STEP_NAME, "metrics.$step_name", "metrics._id", "metrics.ws_name"])
 
-                 # replace our wilcard in request_list with matches
-                 for match in matches:
+                elif key.startswith("hparams."):
+                     matches = self.wildcard_matches_in_list(key, name_list=avail_list, omits=["hparams._id", "hparams.ws_name"])
+
+                elif key.startswith("tags."):
+                     matches = self.wildcard_matches_in_list(key, name_list=avail_list, omits=["tags._id", "tags.ws_name"])
+
+                else:
+                    errors.api_error("wildcards not allowed with this column: {}".format(key))
+
+                actual_cols += matches
+
+                # replace our wildcard in request_list with matches
+                for match in matches:
                     user_name = self.default_user_name(match)
                     new_value = {"user_name": user_name, "user_fmt": None}
-                    new_col_args[match] = new_value
+                    #new_col_args[match] = new_value
+                    new_value["key"] = match
+                    new_col_args.append(new_value)
 
             elif key in avail_list:
                 actual_cols.append(key)
-                new_col_args[key] = value
+                #new_col_args[key] = value
+                value["key"] = key
+                new_col_args.append(value)
         
         return actual_cols, new_col_args
 
@@ -418,18 +435,22 @@ class ReportBuilder():
         started = time.time()
 
         if hide_empty_cols:
+            for col in hide_empty_cols:
+                if not col in user_to_actual:
+                    errors.user_error("unknown column name in hide_empty_cols: {}".format(col))
+
             hide_empty_cols = [user_to_actual[col] for col in hide_empty_cols]
 
         # here is where DATABASE SERVICE does all the hard work for us
         if which == "runs":
             records = database.get_filtered_sorted_run_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
-                count_runs, buffer_size)
+                count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
         elif which == "jobs":
             records = database.get_filtered_sorted_job_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
-                count_runs, buffer_size, hide_empty_cols)
+                count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
         elif which == "nodes":
             records = database.get_filtered_sorted_node_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
-                count_runs, buffer_size)
+                count_runs, buffer_size, hide_empty_cols=hide_empty_cols)
         else:
             errors.internal_error("unrecognized value for which: {}".format(which))
 
@@ -491,9 +512,9 @@ class ReportBuilder():
         all_user_columns = self.get_user_columns(user_columns, args)
 
         # parse out the custom column NAMES and FORMATS provided by the user
-        user_col_args = self.build_user_col_args(all_user_columns)
+        user_col_args_raw = self.build_user_col_args(all_user_columns)
 
-        actual_cols, user_col_args = self.get_requested_cols(user_col_args, avail_list)
+        actual_cols, user_col_args = self.get_requested_cols(user_col_args_raw, avail_list)
         return avail_list, actual_cols, user_col_args
 
     def build_report(self, records, user_columns, report_type, args):
@@ -595,11 +616,14 @@ class ReportBuilder():
                         del record["hp_set"]
 
 
+        # NOTE: user_col_args is a list here, so that we can support multiple instances of same col in long report lines
         avail_list, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, user_columns, args)
 
         # store for later use
         self.user_col_args = user_col_args
 
+        actual_cols = [value["key"] for value in user_col_args]
+
         fn_export = utils.safe_value(args, "export")
         if fn_export:
             fn_ext = os.path.splitext(fn_export)[1]
@@ -609,15 +633,13 @@ class ReportBuilder():
 
             sep_char = "," if fn_ext == ".csv" else "\t"
 
-            col_list = user_col_args.keys()
-            count = self.export_records(fn_export, records, col_list, sep_char)
+            count = self.export_records(fn_export, records, actual_cols, sep_char)
             row_count = count - 1
             line = "report exported to: {} ({} rows)".format(fn_export, row_count)
-            lines += [line]
+            lines = [line]
             was_exported = True
         else:
             number_groups =  utils.safe_value(args, "number_groups")
-            actual_cols = list(user_col_args.keys())
 
             text, row_count = self.build_formatted_table(records, avail_cols=avail_list, col_list=actual_cols, 
                 report_type=report_type, group_by=group_by, number_groups=number_groups, 
@@ -666,7 +688,7 @@ class ReportBuilder():
 
     def build_user_col_args(self, requested_list):
 
-        user_col_args = {}
+        user_col_args = []
 
         for col_spec in requested_list:
             col_name = col_spec
@@ -699,7 +721,7 @@ class ReportBuilder():
             # rebuild prefix_name (must be prefix + col_name)
             prefix_name = prefix + "." + col_name if prefix else col_name
 
-            user_col_args[prefix_name] = {"user_name": user_name, "user_fmt": user_fmt}
+            user_col_args.append( {"key": prefix_name, "user_name": user_name, "user_fmt": user_fmt} )
 
         return user_col_args
 
@@ -771,8 +793,20 @@ class ReportBuilder():
             text = "%.*e" % (significance-1, value)
         return text
 
+    def get_user_col_arg(self, col_name):
+        cdx = None
+
+        for cd in self.user_col_args:
+            if cd["key"] == col_name:
+                cdx = cd
+                break
+
+        return cdx
+
     def build_formatted_table(self, records, avail_cols=None, col_list=None, max_col_width=None, 
-        report_type="run-reports", group_by=None, number_groups=False, indent=None, print_report=False, copy_as_html=False):
+        report_type="run-reports", group_by=None, number_groups=False, indent=None, print_report=False, 
+        copy_as_html=False, skip_lines=None):
+
         '''
         Args:
             records:        a list of dict entries containing data to format
@@ -807,6 +841,7 @@ class ReportBuilder():
             uppercase_hdr_cols = self.config.get(report_type, "uppercase-hdr")
             right_align_num_cols = self.config.get(report_type, "right-align-numeric")
             truncate_with_ellipses = self.config.get(report_type, "truncate-with-ellipses")
+            skip_lines = self.config.get(report_type, "skip-lines")
         else:
             # default when running without config file
             if not max_col_width:
@@ -840,7 +875,7 @@ class ReportBuilder():
 
             # examine all records for determining max col_widths
             if self.user_col_args:
-                user_args = self.user_col_args[col]
+                user_args = self.get_user_col_arg(col)
                 user_col = user_args["user_name"]
                 user_fmt = user_args["user_fmt"] 
             else:
@@ -971,7 +1006,7 @@ class ReportBuilder():
                     text += "\n{} {}:\n".format(group_by, group)
 
                 txt, rc = self.generate_report(col_infos, grecords, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-                    col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision)
+                    col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision, skip_lines=skip_lines)
 
                 # indent report
                 txt = "  " + txt.replace("\n", "\n  ")
@@ -983,7 +1018,7 @@ class ReportBuilder():
         else:
             # UNGROUPED REPORT
             text, row_count = self.generate_report(col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-                col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision)
+                col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision, skip_lines=skip_lines)
 
         if indent:
             text = "\n".join([indent + line for line in text.split("\n")])
@@ -1157,7 +1192,7 @@ class ReportBuilder():
         return color
 
     def generate_report(self, col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-        col_space, duration_col_names, time_col_names, significance, max_fixed_length, float_precision):
+        col_space, duration_col_names, time_col_names, significance, max_fixed_length, float_precision, skip_lines=None):
 
         # process COLUMN HEADERS
         text = ""
@@ -1331,6 +1366,9 @@ class ReportBuilder():
 
             text += "\n"
             row_count += 1
+
+            if skip_lines and (row_count % skip_lines == 0):
+                text += "\n"
         
         # all records processed
         if row_count > 5:
