@@ -6,8 +6,9 @@
 import json
 import math
 import fnmatch
-from datetime import timedelta
+import pandas as pd
 from os import fdopen
+from datetime import timedelta
 
 from xtlib import qfe
 from xtlib import utils
@@ -346,6 +347,11 @@ def build_run_filter_dict(store, ws_name, run_list, user_to_actual, builder, arg
     explict_options = qfe.get_explicit_options()
     explict_options = {key.replace("-", "_"): value for key,value in explict_options.items()}
 
+    # handle properties that are exceptions to needing to be explicit
+    for key in ["child", "parent"]:
+        if (key in args) and (args[key] is not None) and (key not in explict_options):
+            explict_options[key] = args[key]
+
     # # parent is an exception - its default=0 needs to be filtered on
     # if "parent" not in explict_options:
     #     explict_options["parent"] = args["parent"]
@@ -1078,7 +1084,7 @@ def wrapup_runs_for_nodes(store, ws_name, job_id, nodes, max_workers):
         fb.feedback("no runs needing wrapup found")
 
     # finally, mark the job as CANCELLED
-    store.database.set_job_status(ws_name, job_id, "cancelled")
+    #store.database.set_job_status(ws_name, job_id, "cancelled")
 
     fb.feedback("done", is_final=True)
 
@@ -1180,7 +1186,7 @@ def get_run_records_simple(store, ws_name, run_names, fields_dict=None):
     return run_records
 
 def get_run_records(job_id, run_ids,  metric_names=None, filter_dict=None, fields_dict=None, include_hparams=False, include_log_records=None, 
-    workspace_name=None, config=None):
+    workspace_name=None, config=None, include_metrics=True):
 
     # JIT import to prevent circular references
     from xtlib.storage.store import Store
@@ -1197,7 +1203,7 @@ def get_run_records(job_id, run_ids,  metric_names=None, filter_dict=None, field
     if not filter_dict:
         filter_dict = {}
     
-    filter_dict = {"job_id": job_id}
+    filter_dict["job_id"] = job_id
     if run_ids:
         filter_dict["run_name"] = {"$in": run_ids}
 
@@ -1214,11 +1220,17 @@ def get_run_records(job_id, run_ids,  metric_names=None, filter_dict=None, field
         for name in full_metric_names:
             #filter_dict[name] = {"$exists": True}
             fields_dict[name] = 1
+
+    elif include_metrics:
+        # include all metrics
+        fields_dict["metrics"] = 1
     
-    fetch_logs = include_log_records or metric_names
+    fetch_logs = include_log_records   #  or metric_names
     run_records = store.database.get_info_for_runs(workspace_name, filter_dict, fields_dict, include_log_records=fetch_logs)
 
-    if metric_names:
+    if metric_names and include_log_records:
+        step_name = None
+
         for record in run_records:
             log_records = record["log_records"]
             metric_sets = build_metrics_sets(log_records)
@@ -1229,14 +1241,113 @@ def get_run_records(job_id, run_ids,  metric_names=None, filter_dict=None, field
                 msr = [md["records"] for md in metric_sets if metric_name in md["keys"]]
                 if msr:
                     values = msr[0]
-                    steps = [value["step"] for value in values]
+                    
+                    if not step_name:
+                        # extract step name
+                        step_name = values[0][constants.STEP_NAME]
+
+                        # add to last_metrics dict
+                        if "metrics" in record:
+                            record["metrics"][constants.STEP_NAME] = step_name
+
+                    steps = [value[step_name] for value in values]
                     metric_values = [value[metric_name] for value in values]
 
                     metrics_dict[metric_name] = {"steps": steps, "values": metric_values}
 
             # update record
             record["metrics_dict"] = metrics_dict
+
+            # del log records if user didn't explictly include them
             if not include_log_records:
                 del record["log_records"]
 
     return run_records
+
+def flatten_prop(rr, prop):
+    if prop in rr:
+        # remove dict
+        pd = rr[prop]
+        del rr[prop]
+
+        # zap _id column
+        if "_id" in pd:
+            del pd["_id"]
+
+        # add pd key/value pairs to record dict
+        rr.update(pd)
+
+def replicate_run_record_with_metrics_dict(rr, step_name):
+    records_by_step = {}
+
+    md = rr["metrics_dict"]
+    del rr["metrics_dict"]
+
+    for name, kd in md.items():
+        steps = kd["steps"]
+        values = kd["values"]
+
+        for step, value in zip(steps, values):
+            if step in records_by_step:
+                new_record = records_by_step[step]
+            else:
+                new_record = dict(rr)
+                records_by_step[step] = new_record
+
+            new_record[step_name] = step
+            new_record[name] = value
+
+    records = list(records_by_step.values())
+    return records
+
+def get_run_records_as_dataframe(job_id, run_ids=None, metric_names=None, filter_dict=None, fields_dict=None, include_hparams=False, 
+    workspace_name=None, config=None, detail=False, melt=False):
+
+    if not filter_dict:
+        filter_dict = {"is_child": 1}
+
+    run_records = get_run_records(job_id=job_id, run_ids=run_ids, metric_names=metric_names, filter_dict=filter_dict, fields_dict=fields_dict, 
+        include_hparams=include_hparams, include_log_records=detail, workspace_name=workspace_name, config=config)
+
+    # find step name for this set of run_records
+    step_name = None
+
+    if run_records:
+        rr = run_records[0]
+        md = rr["metrics"]
+
+        if constants.STEP_NAME in md:
+            step_name = md[constants.STEP_NAME]
+
+        elif "epoch" in md:
+                step_name == "epoch"
+
+        elif "step" in md:
+            step_name = "step"
+
+        elif config:
+            step_name = config.get("general", "step-name")
+
+    # flatten metrics
+    new_records = []
+
+    for rr in run_records:
+        flatten_prop(rr, "metrics")
+        flatten_prop(rr, "hparams")
+
+        if "log_records" in rr:
+            del rr["log_records"]
+
+        if detail:
+            records = replicate_run_record_with_metrics_dict(rr, step_name)
+            new_records += records
+
+        else:
+            new_records.append(rr)
+
+    df = pd.DataFrame.from_records(new_records)
+
+    if melt:
+        pd.melt(df, step_name)
+
+    return df
