@@ -216,7 +216,7 @@ class Runner():
     def write_node_info_data(self, _id, ws_name, job_id, node_index, total_run_count, node_count, 
         box_name, run_name, secret, service_info, exper_name, args):
 
-        target = args["target"]
+        compute_target = args["target"]
         str_now = utils.get_arrow_time_str()
 
         job_num = job_helper.get_job_number(job_id)
@@ -227,11 +227,14 @@ class Runner():
         first, last = self.core.compute_run_indexes_for_node(total_run_count, node_count, node_index)
         node_run_count = last - first + 1
         node_id = utils.node_id(node_index)
-        service_info_text = json.dumps(service_info)
+        service_info_text = json.dumps(service_info) if service_info else None
+        
+        compute_def = args["compute_def"]
+        aml_compute = utils.safe_value(compute_def, "compute")
 
         dd = {"_id": _id, "ws_name": ws_name, "job_id": job_id, "exper_name": exper_name, 
             "node_name": node_name, "node_id": node_id, "node_index": node_index, "node_num": node_num, 
-            "target": target, "total_runs": node_run_count, "box_name": box_name, "run_name": run_name,
+            "aml_compute": aml_compute, "compute_target": compute_target, "total_runs": node_run_count, "box_name": box_name, "run_name": run_name,
             "secret": secret, "service_info": service_info_text}
 
         # write dd record to node_info
@@ -482,11 +485,11 @@ class Runner():
 
         return script_dir
 
-    def build_docker_cmd(self, docker_name:str, target: str, bootstrap_dir: str, job_secret: str, 
+    def build_docker_cmd(self, docker_name:str, target: str, service_type, bootstrap_dir: str, job_secret: str, 
             setup_name: str, args: List[str]):
         '''
         This is only called for backends that don't have their own support for running in docker.  Currently, this is
-        just the POOL backend.  This function will replace the user's script cmd with:
+        just the POOL backend (and Azure Batch?).  This function will replace the user's script cmd with:
 
             - optional 'docker login' cmd
             - 'docker run' cmd
@@ -527,6 +530,11 @@ class Runner():
         dest_src_dir = "/usr/src"     # "/root/.xt/cwd"
         script_dir = "%CD%" if for_windows else "$PWD"
         mappings = "-v {}:{}".format(script_dir, dest_src_dir)
+
+        if service_type == "batch":
+            # map a virtual docker drive to the azure batch logs directory
+            mappings += " -v $(dirname $PWD):/usr/logs"
+
         options = "--rm"
 
         # collect env vars 
@@ -566,7 +574,7 @@ class Runner():
         options += " --privileged"
 
         # append user options
-        other_options = args["docker_other_options"]
+        other_options = " ".join(args["docker_other_options"])
         if other_options:
             options += " " + other_options
 
@@ -606,11 +614,12 @@ class Runner():
 
         for pp in pip_packages:
             if pp.endswith("==*"):
-                package = pp[:-3]
+                pp_spec = pp.split(" ")[-1]
+                package = pp_spec[:-3]
                 version = self.get_installed_package_version(package)
                 if not version:
                     errors.env_error("version number for specified pip package not found in environment: " + package)
-                pp = package + "==" + version
+                pp = pp.replace("*", version)
 
             new_pip_packages.append(pp)
 
@@ -693,6 +702,8 @@ class Runner():
         python_path = utils.safe_value(setup_def, "python-path")
         other_cmds = utils.safe_value(setup_def, "other-cmds")
         pre_cmds = utils.safe_value(setup_def, "pre-cmds")
+        python_version = utils.safe_value(setup_def, "python-version", "3.6.0")
+        use_legacy_resolver = utils.safe_value(setup_def, "use-legacy-resolver")
         install_blobfuse = utils.safe_value(setup_def, "install-blobfuse")
 
         args["conda_packages"] = conda_packages if conda_packages else []
@@ -700,6 +711,8 @@ class Runner():
         args["python_path"] = python_path if python_path else []
         args["other_cmds"] = other_cmds if other_cmds else []
         args["pre_cmds"] = pre_cmds if pre_cmds else []
+        args["python_version"] = python_version
+        args["use_legacy_resolver"] = use_legacy_resolver
 
         # install blobfuse defaults to True
         args["install_blobfuse"] = True if install_blobfuse is None else install_blobfuse
@@ -751,7 +764,7 @@ class Runner():
                 args["docker"] = docker_name
     
             if docker_name and docker_name != "none":
-                self.build_docker_cmd(docker_name, compute, bootstrap_dir, job_secret, setup_name, args)
+                self.build_docker_cmd(docker_name, compute, service_type, bootstrap_dir, job_secret, setup_name, args)
                 
         # BUILD CMDS (from static hparam search, user multi cmds, or single user cmd)
         cmds, runsets, total_run_count, repeat_count, run_specs, using_hp, using_aml_hparam, sweeps_text, search_style = \
@@ -775,11 +788,10 @@ class Runner():
 
         resume_name = args['resume_name']
         keep_name = False  # args['keep_name']
-        experiment = args['experiment']
         is_distributed = args['distributed']
         direct_run = args["direct_run"]
 
-        # CREATE JOB to hold all runs
+        # CREATE JOB (in database and blobs) to hold all runs
         if fake_submit:
             # use lastrun/lastjob info to get a fast incremental fake job number
             xtd = xt_dict.read_xt_dict()
@@ -791,6 +803,11 @@ class Runner():
             job_id = self.store.create_job(workspace)
         fb.feedback(job_id)
 
+        # expand system names in experiment
+        experiment = args["experiment"]
+        experiment = utils.expand_xt_vars(experiment, job_id=job_id)
+        args["experiment"] = experiment
+
         if experiment:
             # create the experiment, if it doesn't already exist
             if not self.store.does_experiment_exist(workspace, experiment):
@@ -910,7 +927,49 @@ class Runner():
                 fn_dst = code_uploaded_dir + "/" + os.path.basename(fn)
                 shutil.copyfile(fn, fn_dst)
 
-        # SUBMIT JOB 
+        '''
+        Order dependency analysis (updated: Aug-31-2022)
+            - the job_info record is created in the above code
+
+            - log_runs_queued() creates the following node records:
+                - node_info 
+                - node_stats
+                - node_tags 
+
+            - log_runs_queued() calls process_run_event() which creates the following PARENT run records:
+                - run_tags
+                - run_info
+                - run_stats
+
+            - call to store.log_job_info() calls odbc.update_job_info() which:
+                - updates job_info
+                - updates job_stats
+                - creates job_tags
+
+            -backend.submit_job():
+                - submits each node as a job (singularity) or the set of nodes as a pool (azure batch)
+                - as each node is submitted/processed, the associated node_stats record is updated (pool_info column)
+
+            - When a node is started by the backend, the node will need to access:
+                - the associated node_stats record (to update node timings)
+                - the associated job_stats record (to allocate child run id's and update job timings)
+
+            - finally, after all nodes have been started, update_job_info is called to update the
+            columns: service_job_info and service_info_by_node
+
+        '''
+        # NOTE: order of next 3 statements must be maintained (see above).
+        self.log_runs_queued(runs_by_box, workspace, job_id, total_run_count, secrets_by_node, 
+            service_info_by_node=None, args=args)
+
+        #self.store.database.create_job_stats(workspace, job_id)
+
+        # create the JOB_INFO record (and fully initialize JOB_STATS)
+        self.create_job_info(job_id, search_style, total_run_count, runs_by_box, experiment, job_guid, job_secret, repeat_count, 
+            compute_def, workspace, secrets_by_node, args)
+
+        # SUBMIT JOB to backend 
+        # backend requirement: update each node_info ASAP with service_info_by_node
         service_job_info, service_info_by_node = \
             self.backend.submit_job(job_id, job_runs, workspace, compute_def, resume_name, 
                 repeat_count, using_hp, runs_by_box, experiment, code_uploaded_dir, adjustment_scripts, args)
@@ -918,14 +977,31 @@ class Runner():
         # POST SUBMIT processing
         if not fake_submit:
 
-            # mark runs as QUEUED
-            self.log_runs_queued(runs_by_box, workspace, job_id, total_run_count, secrets_by_node, 
-                service_info_by_node, args)
+            # update job_info with: service_job_info, service_info_by_node
+            self.update_job_info(job_id, workspace, service_job_info, service_info_by_node)
+
+        fb.feedback("done", is_final=True)
+
+        escape = args["escape"]
+
+        # update lastrun/lastjob info
+        xtd = xt_dict.read_xt_dict()
+
+        xtd["last_run"] = last_run
+        xtd["last_job"] = job_id
+
+        xt_dict.write_xt_dict(xtd)
+
+        # return values for API support (X)
+        return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, compute_def, job_id 
+
+    def create_job_info(self, job_id, search_style, total_run_count, runs_by_box, experiment, job_guid, job_secret, repeat_count, 
+            compute_def, workspace, secrets_by_node, args):
 
             fb.feedback("logging job")
 
             # write the job info file (now that backend has had a chance to update it)
-            job_num = int(job_id[3:])
+            job_num = job_helper.get_job_number(job_id)
 
             xt_cmd = args["xt_cmd"]
             schedule = args["schedule"]
@@ -975,13 +1051,11 @@ class Runner():
                 "username": args["username"], 
                 "ws_name": workspace, 
                 "xt_cmd": xt_cmd, 
-
-                # cannot add this now (since current design cannot add columns dynamically)
                 "sleep_on_exit": args["sleep_on_exit"],
 
                 # embedded READONLY props (2)
                 "pool_info": compute_def, 
-                "service_job_info": service_job_info, 
+                #"service_job_info": service_job_info, 
 
                 # JOB_STATS: flat UPDATABLE props (8)
                 "completed_runs": 0, 
@@ -995,7 +1069,7 @@ class Runner():
 
                 # SERVICE_INFO: embedded READONLY props BY NODE (2)
                 "runs_by_box": runs_by_box, 
-                "service_info_by_node": service_info_by_node,
+                #"service_info_by_node": service_info_by_node,
                 
                 # CONNECT_INFO: embedded UPDATABLE props (2)
                 "connect_info_by_node": {}, 
@@ -1010,20 +1084,10 @@ class Runner():
 
             self.store.log_job_info(workspace, job_id, dd)
 
-        fb.feedback("done", is_final=True)
+    def update_job_info(self, job_id, workspace, service_job_info, service_info_by_node):
 
-        escape = args["escape"]
-
-        # update lastrun/lastjob info
-        xtd = xt_dict.read_xt_dict()
-
-        xtd["last_run"] = last_run
-        xtd["last_job"] = job_id
-
-        xt_dict.write_xt_dict(xtd)
-
-        # return values for API support (X)
-        return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, compute_def, job_id 
+        dd = {"ws_name": workspace, "job_id": job_id, "service_job_info": service_job_info, "service_info_by_node": service_info_by_node}
+        self.store.log_job_info(workspace, job_id, dd, update_job_stats=False)
 
     def log_run_and_node(self, workspace, box_name, runs_by_box, job_id, total_run_count, secrets_by_node, 
         service_info_by_node, args):
@@ -1036,7 +1100,7 @@ class Runner():
         node_id = utils.node_id(node_index)
 
         box_secret = secrets_by_node[node_id]
-        service_info = service_info_by_node[node_id]
+        service_info = service_info_by_node[node_id] if service_info_by_node else None
 
         ws_name = args["workspace"]
         exper_name = args["experiment"]
