@@ -21,7 +21,7 @@ def _get_hp_config_path_inner(store, ws_name, aggregate_dest, dest_name, hp_conf
         if not filenames and last_chance:
             errors.store_error("Missing hp-config file for job={}".format(dest_name))
     else:
-        _, filenames = store.get_experiment_filenames(workspace, dest_name, hp_config_dir)
+        _, filenames = store.get_experiment_filenames(ws_name, dest_name, hp_config_dir)
         if not filenames and last_chance:
             errors.store_error("Missing hp-config file for experiment={}".format(dest_name))
 
