@@ -24,7 +24,6 @@ from xtlib import file_utils
 from xtlib import run_helper
 from xtlib import store_utils
 from xtlib import run_errors
-from xtlib.storage import mongo_v2
 from xtlib.storage import odbc
 from xtlib.helpers import xt_config
 from xtlib.utils import log_info, log_title
@@ -201,6 +200,8 @@ class StoreV2():
         self.db_name = db_creds["name"]
 
         if self.db_type == "mongo":
+            from xtlib.storage import mongo_v2
+            
             self.database = mongo_v2.MongoDB2(self.db_conn_str, run_cache_dir=run_cache_dir, store=self, 
                 db_options=db_options) 
         
@@ -966,7 +967,7 @@ class StoreV2():
 
     def wrapup_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
-        after_omit_list=None, node_id=None, run_index=None, start_time=None):
+        after_omit_list=None, node_id=None, run_index=None, start_time=None, log_upload_files=False):
 
         utils.log_info("wrapup_run", run_name)
 
@@ -989,7 +990,9 @@ class StoreV2():
                 #console.print("\nprocessing AFTER: ws_name=", ws_name, ", run_name=", run_name, ", output_files=", output_files, ", from_path=", from_path, ", to_path=", to_path)
 
                 try:
-                    copied = self.upload_files_to_run(ws_name, run_name, to_path, output_files, exclude_dirs_and_files=after_omit_list)
+                    copied = self.upload_files_to_run(ws_name, run_name, to_path, output_files, exclude_dirs_and_files=after_omit_list, 
+                        log_files=log_upload_files)
+                        
                     #console.print("AFTER files copied=", copied)
                     copied_files += copied
 
@@ -1016,12 +1019,13 @@ class StoreV2():
             if not is_parent:
                 # tell database this job has a completed run
                 node_index = utils.node_index(node_id)
+                run_was_started = (start_time is not None)
 
                 utils.log_info("calling DATABASE job_run_exit", run_name)
-                self.database.job_run_exit(ws_name, job_id, exit_code)
+                self.database.job_run_exit(ws_name, job_id, exit_code, run_was_started)
 
                 utils.log_info("calling DATABASE node_run_end", run_name)
-                self.database.node_run_end(ws_name, job_id, node_index, exit_code)
+                self.database.node_run_end(ws_name, job_id, node_index, exit_code, run_was_started)
 
     def copy_run_files_to_run(self, ws_name, from_run, run_wildcard, to_run, to_path):
         return self.helper.copy_run_files_to_run(ws_name, from_run, run_wildcard, to_run, to_path)
@@ -1060,12 +1064,13 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.upload_file(run_fn, source_fn)
 
-    def upload_files_to_run(self, ws_name, run_name, run_folder, source_wildcard, exclude_dirs_and_files=[], job_id=None):
+    def upload_files_to_run(self, ws_name, run_name, run_folder, source_wildcard, exclude_dirs_and_files=[], job_id=None, 
+            log_files=False):
         '''upload the local files specified by 'source_wildcard' to the run folder 'run_folder'.
         '''
         #return self.helper.upload_files_to_run(ws_name, run_name, run_folder, source_wildcard, exclude_dirs)
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
-        return rf.upload_files(run_folder, source_wildcard, exclude_dirs_and_files=exclude_dirs_and_files)
+        return rf.upload_files(run_folder, source_wildcard, exclude_dirs_and_files=exclude_dirs_and_files, log_files=log_files)
 
     def read_log_records(self, ws_name, run_name):
         text = self.read_run_file(ws_name, run_name, "run.log")
@@ -1107,12 +1112,12 @@ class StoreV2():
         utils.run_on_threads(thread_worker, id_list, self.max_log_workers, [ws_name])
         return all_run_records
 
-    def download_file_from_run(self, ws_name, run_name, run_fn, dest_fn, job_id=None):
+    def download_file_from_run(self, ws_name, run_name, run_fn, dest_fn, job_id=None, use_snapshot=False):
         '''download the run file 'run_fn' to the local file 'dest_fn'.
         '''
         dest_fn = os.path.abspath(dest_fn)      # ensure it has a directory specified
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
-        return rf.download_file(run_fn, dest_fn)
+        return rf.download_file(run_fn, dest_fn, use_snapshot=use_snapshot)
 
     def download_files_from_run(self, ws_name, run_name, run_wildcard, dest_folder, job_id=None):
         '''download the run files specified by 'run_wildcard' to the local folder 'dest_folder'.
@@ -1170,12 +1175,12 @@ class StoreV2():
     def read_job_info_file(self, ws_name, job_id):
         return self.helper.read_job_info_file(ws_name, job_id)
 
-    def log_job_info(self, ws_name, job_id, dd):
+    def log_job_info(self, ws_name, job_id, dd, update_job_stats=True):
         text = json.dumps(dd, indent=4)
         self.helper.write_job_info_file(ws_name, job_id, text)
 
         # update database-db for fast access
-        self.database.create_db_job(dd)
+        self.database.create_db_job(dd, update_job_stats=update_job_stats)
 
     def log_job_event(self, ws_name, job_id, event_name, data_dict=None, event_time=None):
         if not event_time:
@@ -1318,13 +1323,13 @@ class StoreV2():
 
     # ---- FILES OBJECTS ----
 
-    def root_files(self, root_name, use_blobs=False):
+    def root_files(self, root_name, use_blobs=True):
         return self.helper.root_files(root_name, use_blobs)
 
-    def workspace_files(self, ws_name, use_blobs=False):
+    def workspace_files(self, ws_name, use_blobs=True):
         return self.helper.workspace_files(ws_name, use_blobs)
 
-    def run_files(self, ws_name, run_name, use_blobs=False, job_id=None):
+    def run_files(self, ws_name, run_name, use_blobs=True, job_id=None):
         if job_id is None and self.show_missing_jobid_calls:
 
             console.print("internal warning: job_id is None in call to run_files() [will run slower as a result]")
@@ -1335,10 +1340,13 @@ class StoreV2():
 
         return self.helper.run_files(ws_name, run_name, use_blobs, job_id=job_id)
 
-    def experiment_files(self, ws_name, exper_name, use_blobs=False):
+    def node_files(self, ws_name, node_id, use_blobs=True, job_id=None):
+        return self.helper.node_files(ws_name, job_id, node_id)
+
+    def experiment_files(self, ws_name, exper_name, use_blobs=True):
         return self.helper.experiment_files(ws_name, exper_name, use_blobs)
 
-    def job_files(self, ws_name, job_name, use_blobs=False):
+    def job_files(self, ws_name, job_name, use_blobs=True):
         return self.helper.job_files(ws_name, job_name, use_blobs)
 
 
