@@ -345,8 +345,24 @@ class AzureBlobStore21(implements(StoreInterface)):
             # print("writing to dest_dir: ", dest_dir)
             # print("len(dest_fn)=", len(dest_fn))
 
-            result = self.bs.get_blob_to_path(container, blob_path, dest_fn, snapshot=snapshot, progress_callback=progress_callback)
-            text = result.content
+            try:
+                result = self.bs.get_blob_to_path(container, blob_path, dest_fn, snapshot=snapshot, progress_callback=progress_callback)
+                text = result.content
+
+            except BaseException as ex:
+                if "specified using HTTP conditional header(s) is not met" in str(ex):
+                    # blob changes during read; try again using a snapshot of the blob
+                    print("\nBlob changed during reading; switching to read from snapshot...")
+                    props = self.snapshot_blob(container, blob_path)
+                    snapshot_id = props.snapshot
+                    result = self.bs.get_blob_to_path(container, blob_path, dest_fn, snapshot=snapshot_id, progress_callback=progress_callback)
+                    text = result.content
+                    self.delete_blob(container, blob_path, snapshot=snapshot_id)
+                    print("snapshot successfully read\n")
+                else:
+                    # re-raise the unrecognized exception
+                    raise
+
         else:
             md = blob.metadata
             if "hdi_isfolder" in md and md["hdi_isfolder"]:
