@@ -273,7 +273,7 @@ class StoreBlobObjs():
             
         return len(matches) > 0
 
-    def _upload_files(self, ws_name, ws_path, source_wildcard, recursive=False, exclude_dirs_and_files=[]):
+    def _upload_files(self, ws_name, ws_path, source_wildcard, recursive=False, exclude_dirs_and_files=[], log_files=False):
         copied_files = []
 
         # ensure the container exists
@@ -303,7 +303,11 @@ class StoreBlobObjs():
                 continue
 
             if os.path.isfile(source_fn):
-                console.detail("uploading FILE: " + source_fn)
+                console.detail("uploading AFTER file: " + source_fn)
+                if log_files:
+                    file_size = os.path.getsize(source_fn)
+                    console.print("uploading AFTER file: {} ({:,} bytes)".format(source_fn, file_size))
+
                 blob_path = ws_path + "/" + source_name
                 #console.print("ws_name=", ws_name, ", blob_path=", blob_path, ", source_fn=", source_fn)
                 result = self.provider.create_blob_from_path(ws_name, blob_path, source_fn)
@@ -313,7 +317,7 @@ class StoreBlobObjs():
                 # copy subdir
                 console.detail("uploading DIR: " + source_fn)
                 copied  = self._upload_files(ws_name, ws_path + "/" + source_name, source_fn + "/*", recursive=recursive, 
-                    exclude_dirs_and_files=exclude_dirs_and_files)
+                    exclude_dirs_and_files=exclude_dirs_and_files, log_files=log_files)
                 copied_files = copied_files + copied
 
         #console.print("copied_files=", copied_files)
@@ -866,7 +870,7 @@ class StoreBlobObjs():
 
     # ---- FILES OBJECTS helper functions ----
 
-    def run_files(self, ws_name, run_name, use_blobs=False, job_id=None):
+    def run_files(self, ws_name, run_name, use_blobs=True, job_id=None):
         if use_blobs:
             if not job_id:
                 # here, we support job_id and run_name NOT being tied together for import
@@ -888,25 +892,31 @@ class StoreBlobObjs():
         else:
             errors.general_error("store API only supports blob objects")
 
-    def experiment_files(self, ws_name, exper_name, use_blobs=False):
+    def experiment_files(self, ws_name, exper_name, use_blobs=True):
         if use_blobs:
             return ExperimentBlobs(self, ws_name, exper_name)
         else:
             errors.general_error("store API only supports blob objects")
 
-    def workspace_files(self, ws_name, use_blobs=False):
+    def workspace_files(self, ws_name, use_blobs=True):
         if use_blobs:
             return WorkspaceBlobs(self, ws_name)
         else:
             errors.general_error("store API only supports blob objects")
 
-    def job_files(self, ws_name, job_name, use_blobs=False):
+    def node_files(self, ws_name, job_id, node_id, use_blobs=True):
+        if use_blobs:
+            return NodeBlobs(self, ws_name, job_id, node_id)
+        else:
+            errors.general_error("store API only supports blob objects")
+
+    def job_files(self, ws_name, job_name, use_blobs=True):
         if use_blobs:
             return JobBlobs(self, ws_name, job_name)
         else:
             errors.general_error("store API only supports blob objects")
 
-    def root_files(self, root_name, use_blobs=False):
+    def root_files(self, root_name, use_blobs=True):
         if use_blobs:
             return RootBlobs(self, root_name)
         else:
@@ -960,10 +970,10 @@ class BlobsObject():
 
         return self.store.provider.create_blob_from_path(container, path, source_fn, progress_callback=progress_callback)
 
-    def upload_files(self, folder, source_wildcard, recursive=False, exclude_dirs_and_files=[]):
+    def upload_files(self, folder, source_wildcard, recursive=False, exclude_dirs_and_files=[], log_files=False):
         path = self._expand_path(folder)
         return self.store._upload_files(self.container, path, source_wildcard, recursive=recursive, 
-            exclude_dirs_and_files=exclude_dirs_and_files)
+            exclude_dirs_and_files=exclude_dirs_and_files, log_files=log_files)
 
     def _process_parent_path(self, container, base_path, path):
         if path.startswith("./"):
@@ -1182,6 +1192,13 @@ class RunBlobs(BlobsObject):
 
         self.base_path = "jobs/{}/runs/{}".format(job_id, run_name)
 
+class NodeBlobs(BlobsObject):
+    def __init__(self, store, ws_name, job_id, node_id):
+        self.store = store
+        self.container = ws_name
+
+        self.base_path = "jobs/{}/nodes/{}".format(job_id, node_id)
+
 class ExperimentBlobs(BlobsObject):
     def __init__(self, store, ws_name, exper_name):
         self.store = store
