@@ -50,6 +50,10 @@ class MongoDB(implements(DbInterface)):
         if not mongo_conn_str:
             errors.internal_error("Cannot initialize MongoDB() with a empty mongo_conn_str")
 
+        # debug only (show version of pymongo being used)
+        # os.system("pip list | grep pymongo")
+        # os.system("conda info")
+
         self.mongo_conn_str = mongo_conn_str
         self.run_cache_dir = run_cache_dir
         self.mongo_client = None
@@ -71,6 +75,9 @@ class MongoDB(implements(DbInterface)):
         self.update_run_stats = True
         self.add_log_records = True
 
+        # this is used to return row_count of available records for first/last queries
+        self.avail_row_count = None
+
     #---- UTILS ----
 
     # API call
@@ -219,6 +226,21 @@ class MongoDB(implements(DbInterface)):
     def get_next_end_id(self, ws_name, default_next_run=1):
         return self.get_next_sequential_ws_id(ws_name, "next_end", default_next_run)
 
+    def get_next_end_id_without_update(self, ws_name):
+
+        db = self.mongo_db
+        filter_dict = {"_id": ws_name}
+        fields_dict = {"next_end": 1}
+
+        records = self.mongo_with_retries("get_next_end_id_without_update", lambda: \
+            db.ws_counters.find(filter_dict, fields_dict).limit(1), return_records=True)
+
+        #print("records=", records, ", ws_name=", ws_name, ", filter=", filter_dict, ", fields=", fields_dict)
+        next_end = utils.safe_cursor_value(records, "next_end")
+        assert next_end >= 1
+
+        return next_end
+
     def mongo_with_retries(self, name, mongo_cmd, ignore_error=False, return_records=False):
         import pymongo.errors
 
@@ -630,6 +652,11 @@ class MongoDB(implements(DbInterface)):
 
         # put our mongo operations together in a retry-compatible function
         def fetch():
+            # first query is for available count
+            cursor = self.mongo_db[container].find(filter_dict, col_dict)
+            self.avail_row_count = cursor.count()
+
+            # second query is for actual records
             cursor = self.mongo_db[container].find(filter_dict, col_dict)
             if sort_col:
                 cursor = cursor.sort(sort_col, sort_dir)
