@@ -310,6 +310,7 @@ def get_python_value_from_text(part):
         native python value
     '''
     part = part.strip()
+    
     if part.isdigit():
         value = int(part)
     elif str_is_float(part):
@@ -324,6 +325,30 @@ def get_python_value_from_text(part):
 
     return value        
 
+def get_python_value_from_possible_text_list(input):
+    if isinstance(input, str):
+        output = get_python_value_from_text(input)
+
+    elif isinstance(input, list):
+        types = {}
+        values = []
+        for part in input:
+            value = get_python_value_from_text(part)
+            values.append(value)
+            tn = type(value).__name__
+            types[tn] = 1
+
+        # make types of values consistent
+        if "str" in types:
+            values = [str(value) for value in values]
+
+        elif "float" in types:
+            values = [float(value) for value in values]
+
+        output = values
+
+    return output
+
 def get_arrow_time_str():
     ''' 
     returns NOW as an arrow time string
@@ -859,3 +884,12 @@ def expand_xt_vars(text, job_runs=None, run_id=None, job_id=None, node_index=0,
 
     return text
 
+def debug_path(path):
+    while path and len(path) > 3:
+        exists = os.path.exists(path)
+        is_file = os.path.isfile(path)        
+        is_dir = os.path.isdir(path)
+        console.print("path: {}, exists: {},       is_file: {},      is_dir: {}".format(path, exists, is_file, is_dir))
+
+        path = os.path.dirname(path)
+
