@@ -78,6 +78,39 @@ def get_fn_local_config(args):
 
     return fn
 
+def process_template(config, args):
+    template_name = args[0][1:]
+    args = args[1:]
+
+    td = config.get("templates", template_name)
+    if not td:
+        errors.syntax_error("Template not found: {}".format(template_name))
+
+    hidden = utils.safe_value(td, "hidden", False)
+    if hidden:
+        errors.syntax_error("cannot use a template that is marked as hidden: {}".format(template_name))
+
+    templ = td["command"]
+    targs = templ.split()
+    tats = [a for a in targs if a.startswith("@")]
+
+    if len(tats) > len(args):
+        errors.syntax_error("templates requires {} args, but only {} were supplied".format(len(tats), len(args)))
+
+    for i, targ in enumerate(targs):
+        if targ.startswith("@"):
+            targs[i] = args.pop(0)
+
+    cmd = " ".join(targs)
+
+    # add any remaining args to the end of the command
+    cmd += " " + " ".join(args)
+
+    print("template expansion: {}".format(cmd))
+    args = cmd.split()
+
+    return args
+
 def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False, raise_syntax_exception=True):
     '''
     Parse and execute the specified cmd.
@@ -131,9 +164,9 @@ def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False,
         if name == "help":
             impl.set_basic_mode(basic_mode)
 
-    # this enables QFE to match a function by its module name, to the class instance to process the command
-    # impl_dict = {"xtlib.impl_utilities": utilities, "xtlib.impl_storage": storage, 
-    #     "xtlib.impl_compute": compute, "xtlib.impl_help": help_impl}
+    # process template invocations
+    if args and args[0].startswith("@"):
+        args = process_template(config, args)
 
     # this parses args and calls the correct command function with its args and options correctly set.
     # the config object supplies the default value for most options and flags.
