@@ -70,6 +70,8 @@ def run_cmd_on_server(cmd_dict, retry_count):
 def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False):
 
     utils.init_logging(constants.FN_XT_EVENTS, logger, "XT session")
+    # print("cmd=", cmd)
+    # print("sys.argv[1:]", sys.argv[1:])
 
     # fix artifact if no args passed, we end up with python's first arg
     if cmd:
@@ -83,6 +85,19 @@ def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False)
         orig_args = sys.argv[1:]
         console.diag("orig_args={}".format(orig_args))
 
+    # add back quotes where needed 
+    for a, arg in enumerate(orig_args):
+        if " " in arg:
+            if "=" in arg:
+                name, value = arg.split("=")
+                # ensure it is not already quoted
+                if value and not value[0] in "'`\"":
+                    orig_args[a] = "{}='{}'".format(name, value)
+
+            # ensure it is not already quoted
+            elif arg and not arg[0] in "'`\"":
+                orig_args[a] = "'" + arg + "'"
+
     cmd = " ".join(orig_args)
     cmd = cmd.strip()
 
@@ -95,6 +110,7 @@ def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False)
 
     mid_elapsed = time.time() - xt_start_time
     #console.print("mid_elapsed={:.2f}".format(mid_elapsed))
+    #print("sys.argv[1:]", sys.argv[1:])
     
     if not use_server or disable_quickstart:
         # NORMAL start-up mode
