@@ -53,6 +53,8 @@ class XTVault():
         if not os.path.exists(fn_keys):
             fn_keys = os.path.expanduser("~/.xt/stores/{}/keys.bin".format(self.store_name))
 
+        console.diag("looking for grok_creds: {}".format(fn_keys))
+
         if os.path.exists(fn_keys):
             # GROK server creds
             creds = file_utils.read_text_file(fn_keys)  
@@ -113,8 +115,8 @@ class XTVault():
                     console.print("authenticating with azure thru browser... ", flush=True, end="")
                     used_browser = True
                 
-                cpo = TokenCachePersistenceOptions()
-                credential = InteractiveBrowserCredential(cache_persistence_options=cpo,
+                cpo = TokenCachePersistenceOptions(allow_unencrypted_storage=True)
+                credential = InteractiveBrowserCredential(cache_persistence_options=cpo, 
                     authentication_record=deserialized_record)
 
             elif authentication == "device-code":
